from numpy import arcsin, exp, sqrt


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotW23
        A SlotW23 object

    Returns
    -------
    point_dict: dict
        A dict of the slot point coordinates
    """
    Rbo = self.get_Rbo()
    if self.is_cstt_tooth:
        # Compute W1 and W2 to match W3 tooth constraint
        self._comp_W()

    # alpha is the angle to rotate Z0 so ||Z1,Z8|| = W0
    alpha = float(arcsin(self.W0 / (2 * Rbo)))

    # comp point coordinate (in complex)
    Z0 = Rbo * exp(1j * 0)
    Z1 = Z0 * exp(-1j * alpha)

    if self.is_outwards():
        Z2 = (Rbo + self.H0) * exp(-1j * alpha)
        Z3 = Z2.real + self.H1 - 1j * self.W1 / 2

        H2 = sqrt(self.H2 ** 2 - ((self.W2 - self.W1) / 2.0) ** 2)
        Z4 = Z3.real + H2 - 1j * self.W2 / 2
    else:  # inward slot
        Z2 = (Rbo - self.H0) * exp(-1j * alpha)
        Z3 = Z2.real - self.H1 - 1j * self.W1 / 2

        H2 = sqrt(self.H2 ** 2 - ((self.W2 - self.W1) / 2.0) ** 2)
        Z4 = Z3.real - H2 - 1j * self.W2 / 2

    point_dict = dict()
    # symetry
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["Z3"] = Z3
    point_dict["Z4"] = Z4
    point_dict["Z5"] = Z4.conjugate()
    point_dict["Z6"] = Z3.conjugate()
    point_dict["Z7"] = Z2.conjugate()
    point_dict["Z8"] = Z1.conjugate()
    return point_dict



# -*- coding: utf-8 -*-

from numpy import pi


def get_angle(self, is_deg=False):
    """Return the angle of the arc

    Parameters
    ----------
    self : Arc2
        An Arc2 object
    is_deg: bool
        True to convert to degree

    Returns
    -------
    angle: float
        Angle of the arc
    """

    if is_deg:
        return self.angle * 180 / pi
    else:
        return self.angle



# -*- coding: utf-8 -*-
from numpy import exp

from ....Classes.Segment import Segment
from ....Classes.Arc1 import Arc1
from ....Classes.SurfLine import SurfLine
from ....Methods import NotImplementedYetError


def get_surface_active(self, alpha=0, delta=0):
    """Return the full winding surface

    Parameters
    ----------
    self : SlotW10
        A SlotW10 object
    alpha : float
        float number for rotation (Default value = 0) [rad]
    delta : complex
        complex number for translation (Default value = 0)

    Returns
    -------
    surf_wind: Surface
        Surface corresponding to the Winding Area
    """
    # get the name of the lamination
    st = self.get_name_lam()

    # Compute point
    Rbo = self.get_Rbo()
    alpha_op = self.comp_angle_opening()
    R0 = ((self.W0 / 2) ** 2 + self.H0 ** 2) / (2 * self.H0)

    Z1 = Rbo * exp(-1j * alpha_op / 2)
    Z2 = Rbo * exp(1j * alpha_op / 2)
    Ztan1 = Rbo
    if self.is_outwards():
        Ztan2 = (Z1 + Z2) / 2 + self.H0
    else:
        Ztan2 = (Z1 + Z2) / 2 - self.H0

    # Create surface
    curve_list = list()
    if self.is_outwards():
        curve_list.append(Arc1(begin=Z1, end=Z2, radius=-R0, is_trigo_direction=True))
    else:
        curve_list.append(Arc1(begin=Z1, end=Z2, radius=R0, is_trigo_direction=False))
    curve_list.append(Arc1(begin=Z2, end=Z1, radius=-Rbo, is_trigo_direction=False))
    surf = SurfLine(
        line_list=curve_list,
        label="Wind_" + st + "_R0_T0_S0",
        point_ref=(Ztan1 + Ztan2) / 2,
    )

    # Apply transformation
    surf.rotate(alpha)
    surf.translate(delta)

    return surf



# -*- coding: utf-8 -*-


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotM16
        A SlotM16 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    return self.W1 * self.H1



# -*- coding: utf-8 -*-
from ....Classes.Arc3 import Arc3


def get_lines(self):
    """Get the lines needed to draw the surface (2 Arc3)

    Parameters
    ----------
    self : Circle
        A circle Object

    Returns
    -------
    line_list: list
        Two arcs to draw the circle

    """
    # check if the circle is correct
    self.check()
    begin = self.center + self.radius
    end = self.center - self.radius
    # first half-circle
    line1 = Arc3(begin, end, True, label=self.line_label)
    # second half-circle
    line2 = Arc3(end, begin, True, label=self.line_label)
    return [line1, line2]



from numpy import arcsin, exp, angle
from ....Functions.Geometry.comp_flower_arc import comp_flower_arc


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotM14
        A SlotM14 object

    Returns
    -------
    point_dict: dict
        A dict of the slot coordinates
    """

    Rbo = self.get_Rbo()

    Z1 = Rbo * exp(-1j * self.W0 / 2)
    Z4 = Rbo * exp(+1j * self.W0 / 2)

    if self.is_outwards():
        Z2 = (Rbo + self.H0) * exp(-1j * self.W0 / 2)
        Z3 = (Rbo + self.H0) * exp(+1j * self.W0 / 2)
        ZM1 = (Rbo + self.H0) * exp(-1j * self.Wmag / 2)
        ZM4 = (Rbo + self.H0) * exp(+1j * self.Wmag / 2)
        (alpha_lim, ZM3, ZM2) = comp_flower_arc(
            abs(angle(ZM1) - angle(ZM4)), self.Rtopm, abs(ZM1) - self.Hmag
        )
        ZM0 = Rbo + self.H0 - self.Hmag
        Zc = ZM0 + self.Rtopm
    else:
        Z2 = (Rbo - self.H0) * exp(-1j * self.W0 / 2)
        Z3 = (Rbo - self.H0) * exp(+1j * self.W0 / 2)
        ZM1 = (Rbo - self.H0) * exp(-1j * self.Wmag / 2)
        ZM4 = (Rbo - self.H0) * exp(+1j * self.Wmag / 2)
        (alpha_lim, ZM3, ZM2) = comp_flower_arc(
            abs(angle(ZM1) - angle(ZM4)), self.Rtopm, abs(ZM1) + self.Hmag
        )
        ZM0 = Rbo - self.H0 + self.Hmag
        Zc = ZM0 - self.Rtopm

    point_dict = dict()
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["ZM1"] = ZM1
    point_dict["ZM2"] = ZM2
    point_dict["ZM0"] = ZM0
    point_dict["Zc"] = Zc
    # symetry
    point_dict["Z3"] = Z3
    point_dict["Z4"] = Z4
    point_dict["ZM3"] = ZM3
    point_dict["ZM4"] = ZM4

    return point_dict



# -*- coding: utf-8 -*-

from matplotlib.pyplot import gcf
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QDialog, QMessageBox, QWidget

from .....Classes.LamHole import LamHole
from .....Classes.Lamination import Lamination
from .....Classes.LamSlotMag import LamSlotMag
from .....GUI import gui_option
from .....GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.DAVDuct import DAVDuct
from .....GUI.Dialog.DMachineSetup.SLamParam.Gen_SLamParam import Gen_SLamParam


class SLamParam(Gen_SLamParam, QWidget):
    """Step to setup the main lamination parameters"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for the DMachineSetup nav
    step_name = "Lamination"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the widget according to machine

        Parameters
        ----------
        self : SLamParam
            A SLamParam widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        # Set FloatEdit unit
        self.lf_L1.unit = "m"
        self.lf_Wrvd.unit = "m"
        self.u = gui_option.unit
        self.unit_L1.setText(self.u.get_m_name())
        self.unit_Wrvd.setText(self.u.get_m_name())

        # Get the correct object to set
        if self.is_stator:
            self.obj = machine.stator
            if self.obj.Kf1 is None:
                self.obj.Kf1 = 0.95  # Defaut value
        else:
            self.obj = machine.rotor
            if self.obj.L1 is None:
                # Default value for rotor is the stator one
                self.obj.L1 = self.machine.stator.L1
            if self.obj.Kf1 is None:
                # Default value for rotor is the stator one
                self.obj.Kf1 = self.machine.stator.Kf1
        if self.obj.axial_vent is None:
            self.obj.axial_vent = list()

        self.w_mat.update(self.obj, "mat_type", self.matlib)

        self.lf_L1.setValue(self.obj.L1)
        self.lf_Kf1.setValue(self.obj.Kf1)

        # Ventilation setup
        self.update_avd_text()
        if len(self.obj.axial_vent) > 0 and self.obj.axial_vent[0].Zh > 0:
            self.g_ax_vent.setChecked(True)

        if self.obj.Nrvd is None or self.obj.Nrvd == 0:
            self.si_Nrvd.setValue(0)
        else:
            self.si_Nrvd.setValue(self.obj.Nrvd)
            self.g_rad_vent.setChecked(True)

        if self.obj.Wrvd is None or self.obj.Wrvd == 0:
            self.lf_Wrvd.setValue(0)
        else:
            self.lf_Wrvd.setValue(self.obj.Wrvd)
            self.g_rad_vent.setChecked(True)

        self.update_lenght()  # Update out_length if possible

        # Connecting the signal
        self.lf_L1.editingFinished.connect(self.set_L1)
        self.lf_Kf1.editingFinished.connect(self.set_Kf1)
        self.g_rad_vent.toggled.connect(self.enable_rad_vent)
        self.g_ax_vent.toggled.connect(self.enable_ax_vent)
        self.b_axial_duct.clicked.connect(self.set_avd)
        self.si_Nrvd.editingFinished.connect(self.set_Nrvd)
        self.lf_Wrvd.editingFinished.connect(self.set_Wrvd)
        self.b_plot.clicked.connect(self.s_plot)
        self.w_mat.saveNeeded.connect(self.emit_save)

    def emit_save(self):
        self.saveNeeded.emit()

    def set_L1(self):
        """Signal to update the value of L1 according to the line edit

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        """
        if self.lf_L1.value() != self.obj.L1:
            self.obj.L1 = self.lf_L1.value()
            self.update_lenght()
            # Notify the machine GUI that the machine has changed
            self.saveNeeded.emit()

    def set_Kf1(self):
        """Signal to update the value of Kf1 according to the line edit

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        """
        self.obj.Kf1 = self.lf_Kf1.value()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def enable_rad_vent(self, is_checked):
        """Clear radial vent value if g_rad_vent is unselected

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        is_checked : bool
            State of the g_rad_vent checkbox
        """
        if not is_checked:
            self.lf_Wrvd.setValue(0)
            self.si_Nrvd.setValue(0)
            self.set_Nrvd()
            self.set_Wrvd()
            self.update_lenght()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def enable_ax_vent(self, is_checked):
        """Clear axial vent value if g_ax_vent is unselected

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        is_checked : bool
            State of the g_ax_vent checkbox
        """
        if not is_checked:
            self.obj.axial_vent = list()  # Default empty
            self.update_avd_text()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_avd(self):
        """Open the GUI to allow the edition of the axial ventilation duct

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        """
        self.avd_win = DAVDuct(self.obj)
        return_code = self.avd_win.exec_()
        if return_code == QDialog.Accepted:
            # self.obj.axial_vent = self.avd_win.vent
            self.update_avd_text()
            # Notify the machine GUI that the machine has changed
            self.saveNeeded.emit()

    def update_avd_text(self):
        """Update the text with the current number of axial vent

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        """
        Nset = 0
        Nduct = 0
        for vent in self.obj.axial_vent:
            Nset += 1
            Nduct += vent.Zh
        self.out_axial_duct.setText(
            self.tr("Axial: ", "Part of Axial: X set (Y ducts)")
            + str(Nset)
            + self.tr(" set (", "Part of Axial: X set (Y ducts)")
            + str(Nduct)
            + self.tr(" ducts)", "Part of Axial: X set (Y ducts)")
        )

    def set_Nrvd(self):
        """Signal to update the value of Nrvd according to the spinbox

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        """
        self.obj.Nrvd = self.si_Nrvd.value()
        self.update_lenght()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Wrvd(self):
        """Signal to update the value of Wrvd according to the line edit

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        """
        self.obj.Wrvd = self.lf_Wrvd.value()
        self.update_lenght()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def update_lenght(self):
        """Update the text of out_length

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        """
        if self.obj.is_stator:  # Adapt the text to the current lamination
            lam_txt = self.tr("Stator total length = L1 + Nrvd * Wrvd = ")
        else:
            lam_txt = self.tr("Rotor total length = L1 + Nrvd * Wrvd = ")

        if self.obj.L1 is None or self.obj.Nrvd is None or self.obj.Wrvd is None:
            self.out_length.setText(lam_txt + "?")
        else:
            length = format(
                self.u.get_m(self.obj.L1 + self.obj.Nrvd * self.obj.Wrvd), ".4g"
            )
            self.out_length.setText(lam_txt + length + " " + self.u.get_m_name())

    @staticmethod
    def check(lamination):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        lamination : Lamination
            Lamination to check

        Returns
        -------
        error : str
            Error message (return None if no error)
        """

        # Check that everything is set
        try:
            if lamination.L1 is None:
                return "You must set L1 !"
            elif lamination.Kf1 is None:
                return "You must set Kf1 !"
        except Exception as e:
            return str(e)

    def check_gui(self):
        """Check that the widget are set right according to the current machine

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        """
        if self.g_rad_vent.isChecked() and self.obj.Nrvd is None:
            return self.tr("You must set Nrvd or unchecked " "radial Ventilation !")
        if self.g_rad_vent.isChecked() and self.obj.Wrvd is None:
            return self.tr("You must set Wrvd or unchecked " "radial Ventilation !")
        if self.g_ax_vent.isChecked() and len(self.obj.axial_vent) == 0:
            return self.tr(
                "You must add some Axial Ventilation Ducts or "
                "unchecked Axial Ventilation !"
            )

        if not self.g_rad_vent.isChecked():
            self.obj.Nrvd = 0
            self.obj.Wrvd = 0
        if not self.g_ax_vent.isChecked():
            self.obj.axial_vent = list()  # Default empty

    def s_plot(self):
        """Plot the lamination (radial and axial) if possible

        Parameters
        ----------
        self : SLamParam
            A SLamParam object
        """

        # We use an empty lamination for the plot to avoid problem with magnet
        # or winding which are not setup yet
        if self.obj.Rint is None:
            QMessageBox().critical(
                self,
                self.tr("Error"),
                self.tr("Unable to plot the lamination " "(Rint missing)"),
            )
        elif self.obj.Rext is None:
            QMessageBox().critical(
                self,
                self.tr("Error"),
                self.tr("Unable to plot the lamination " "(Rext missing)"),
            )
        else:
            plot_obj = Lamination(
                Rint=self.obj.Rint, Rext=self.obj.Rext, axial_vent=self.obj.axial_vent
            )
            if self.obj.is_stator is not None:
                plot_obj.is_stator = self.obj.is_stator
            plot_obj.plot()
            gcf().canvas.set_window_title(
                self.tr("Lamination (without slots) Front view")
            )



# -*- coding: utf-8 -*-
from ....Classes.LossModel import LossModel
from ....Methods.Simulation.Loss import LossError

"""
The loss models are stored in a list 'model_list'. The losses are categorized by the
respective machine part_label 'part_label', e.g. 'Stator', 'Rotor' or 'Bearings'. It has to 
corespond to the first part_label of the name of the respective FEA group for FEA based loss
models to form the complete FEA group name together with the models 'group' property.

To organize the stored models the 'model_list' is supplemented by 'model_index'
property. 'model_index' is a double stacked dict, where the first layer is for
'part_label', the 2nd layeris for an optional index (if one wants to e.g. have multiple 
stator core loss models). The reason to use a dict with int keys is to have a kind
of sparse list.

To retrieve the model from the 'model_list' the 'model_index' stores the respective
list index where the model is added.

"""
# TODO maybe use machine part_label directly, e.g. use 'machine.stator'
#      and request name from the part_label
def add_model(self, model, part_label, index=None):
    """Add a loss model to the list of loss models. For further details on
    Loss organization see Loss.add_model() comments.

    Parameter
    ---------
    self : Loss
        Loss object

    model : LossModel
        the model to add to the list of loss models

    part_label : str
        Label of the machine part to apply the model, e.g. "Stator" or "Rotor"

    index : int
        explicit index of the model, e.g. if there are multiple 'Stator core'
        loss models one may give an index to each of them

    Return
    ------

    """
    logger = self.get_logger()

    # check type of index
    if not isinstance(index, int) and index is not None:
        logger.warning(
            "Loss.add_model(): Input argument 'index' "
            + f"is of type {type(index).__name__} but should be of type int."
            + "Index set to None."
        )

    # check type of model
    if not isinstance(model, LossModel):
        raise LossError("Input argument 'model' has to be of type 'LossModel'.")

    # get actual index for model
    mdl_index = len(self.model_list)

    # check if dict key exists
    if part_label not in self.model_index.keys():
        self.model_index[part_label] = {}

    # get index if it is not specified
    if index is None:
        keys = [key for key in self.model_index[part_label].keys()]
        if not keys:  # if there is no index, set index to zero
            index = 0
        else:  # if there are other models increment max index by one
            index = max(keys) + 1

    # store the model and the index
    if index in self.model_index[part_label].keys():
        # override model by request
        ii = self.model_index[part_label][index]
        logger.info(
            f"Loss.add_model(): Loss model {self.model_list[ii].name}' "
            + f"with index {index} for {part_label}"
            + f"overriden by model '{model.name}''."
        )
        self.model_list[ii] = model
    else:
        # append model
        self.model_list.append(model)
        self.model_index[part_label][index] = mdl_index



# -*- coding: utf-8 -*-

from numpy import angle, arcsin, arctan, array, cos, exp


def comp_radius(self):
    """Compute the radius of the min and max circle that contains the hole

    Parameters
    ----------
    self : HoleM58
        A HoleM58 object

    Returns
    -------
    (Rmin,Rmax): tuple
        Radius of the circle that contains the hole [m]
    """
    Rext = self.get_Rext()

    Rmax = Rext - self.H1
    Rmin = Rext - self.H0 - self.H2

    return (Rmin, Rmax)



# -*- coding: utf-8 -*-

from numpy import pi
from PySide2.QtCore import Signal
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QWidget, QDialog
from PySide2.QtWidgets import QMessageBox
from PySide2.QtCore import Qt
from ......Functions.load import load
from ......Classes.HoleUD import HoleUD
from ......GUI import gui_option
from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleMUD.Ui_PHoleMUD import Ui_PHoleMUD
from ......GUI.Dialog.DMatLib.MatLib import MatLib
from ......Methods.Slot.Slot import SlotCheckError
from ......GUI.Dxf.DXF_Hole import DXF_Hole
from ......GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect


class PHoleMUD(Ui_PHoleMUD, QWidget):
    """Page to set the Hole from DXF"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for WHoleMag
    hole_name = "Import from DXF"
    hole_type = HoleUD

    def __init__(self, hole=None, matlib=MatLib()):
        """Initialize the widget according to hole

        Parameters
        ----------
        self : PHoleMUD
            A PHoleMUD widget
        hole : HoleUD
            current hole to edit
        matlib : MatLib
            Material Library
        """
        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Set properties
        self.matlib = matlib
        self.hole = hole
        self.u = gui_option.unit
        self.w_mat_dict = dict()  # For magnet materials

        # Setup void material
        self.w_mat_0.setText("mat_void")
        self.w_mat_0.def_mat = "Air"
        self.w_mat_0.is_hide_button = True

        # Setup Path selector for Json files
        self.w_path_json.obj = None
        self.w_path_json.param_name = None
        self.w_path_json.verbose_name = "Load from json"
        self.w_path_json.extension = "JSON file (*.json)"
        self.w_path_json.update()

        # Update the GUI according to the current hole
        self.update_mag_list()
        self.update_graph()
        self.comp_output()

        # Connect the signals
        self.b_dxf.clicked.connect(self.open_dxf_hole)
        self.w_path_json.pathChanged.connect(self.load_hole)

    def update_mag_list(self):
        """Update the material selector list according to hole magnet"""
        # Set void material
        self.w_mat_0.update(self.hole, "mat_void", self.matlib)
        # Remove previous widget:
        for wid in self.w_mat_dict.values():
            self.g_mat_layout.removeWidget(wid)
            wid.setParent(None)
        self.w_mat_dict = dict()
        # Update Magnet materials
        magnet_dict = self.hole.magnet_dict
        for key, mag in magnet_dict.items():
            index = key.split("_")[-1]
            # Add widget
            self.w_mat_dict[key] = WMatSelect(self.g_mat)
            self.w_mat_dict[key].setObjectName(u"w_mat_" + str(index))
            self.g_mat_layout.addWidget(self.w_mat_dict[key])
            # Setup Widget
            self.w_mat_dict[key].setText("Magnet " + str(index) + "")
            self.w_mat_dict[key].def_mat = "MagnetPrius"
            self.w_mat_dict[key].is_hide_button = True
            self.w_mat_dict[key].update(
                self.hole.magnet_dict["magnet_" + str(index)], "mat_type", self.matlib
            )

    def update_graph(self):
        """Plot the lamination with/without the hole"""
        # Use a copy to avoid changing the main object
        lam = self.hole.parent.copy()
        try:
            self.hole.check()
            lam.hole = [self.hole]
        except SlotCheckError:
            # Plot only the lamination
            lam.hole = list()
        # Plot the lamination in the viewer fig
        lam.plot(fig=self.w_viewer.fig, is_show_fig=False)

        # Update the Graph
        self.w_viewer.axes.set_axis_off()
        self.w_viewer.axes.axis("equal")
        if self.w_viewer.axes.get_legend() is not None:
            self.w_viewer.axes.get_legend().remove()
        self.w_viewer.draw()

    def load_hole(self):
        """Load the selected json file and display the hole"""
        # Check that the json file is correct
        try:
            hole = load(self.w_path_json.get_path())
        except Exception as e:
            QMessageBox().critical(
                self,
                self.tr("Error"),
                self.tr("Error when loading file:\n" + str(e)),
            )
            return
        # Check that the json file contain a HoleUD
        if not isinstance(hole, HoleUD):
            QMessageBox().critical(
                self,
                self.tr("Error"),
                self.tr(
                    "The choosen file is not a HoleUD file (" + str(type(hole)) + ")"
                ),
            )
            return

        # Update the hole object
        Zh = self.hole.Zh
        parent = self.hole.parent
        self.hole.__init__(init_dict=hole.as_dict())  # keep pointer
        self.hole.Zh = Zh
        self.hole.parent = parent

        # Update the new GUI according to the slot
        self.update_graph()
        self.update_mag_list()
        self.comp_output()

    def open_dxf_hole(self):
        """Open the GUI to define the HoleUD"""
        # Init GUI with lamination parameters
        self.dxf_gui = DXF_Hole(
            Zh=self.hole.Zh, Lmag=self.hole.parent.L1, lam=self.hole.parent
        )
        self.dxf_gui.setWindowFlags(Qt.Window)  # To maximize the GUI
        self.dxf_gui.show()
        # Update the hole when saving
        self.dxf_gui.accepted.connect(self.set_dxf_path)

    def set_dxf_path(self):
        """Update the hole according to the file defined by DXF_Hole"""
        # Get the saving path from DXF_Hole
        self.w_path_json.set_path_txt(self.dxf_gui.save_path)
        # Update hole and GUI
        self.load_hole()
        self.dxf_gui = None

    def comp_output(self):
        """Compute and display the hole output

        Parameters
        ----------
        self : PHoleMUD
            A PHoleMUD widget
        """
        is_set = False
        if self.check() is None:
            try:
                # We compute the output only if the hole is correctly set
                # Compute all the needed output as string
                s_surf = format(self.u.get_m2(self.hole.comp_surface()), ".4g")
                m_surf = format(self.u.get_m2(self.hole.comp_surface_magnets()), ".4g")
                (Rmin, Rmax) = self.hole.comp_radius()
                Rmin_txt = format(self.u.get_m(Rmin), ".4g")
                Rmax_txt = format(self.u.get_m(Rmax), ".4g")

                # Update the GUI to display the Output
                self.out_slot_surface.setText(
                    "Hole full surface : " + s_surf + " " + self.u.get_m2_name()
                )
                self.out_magnet_surface.setText(
                    "Hole magnet surface : " + m_surf + " " + self.u.get_m2_name()
                )
                self.out_Rmin.setText("Rmin : " + Rmin_txt + " " + self.u.get_m_name())
                self.out_Rmax.setText("Rmax : " + Rmax_txt + " " + self.u.get_m_name())
                is_set = True
            except:
                pass

        if not is_set:
            # We can't compute the output => We erase the previous version
            # (that way the user know that something is wrong)
            self.out_slot_surface.setText("Hole full surface : ?")
            self.out_magnet_surface.setText("Hole magnet surface : ?")
            self.out_Rmin.setText("Rmin : ?")
            self.out_Rmax.setText("Rmax : ?")

    def check(self):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        self : PHoleMUD
            A PHoleMUD widget

        Returns
        -------
        error : str
            Error message (return None if no error)
        """

        # Constraints and None
        try:
            self.hole.check()
        except SlotCheckError as error:
            return str(error)

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()



# -*- coding: utf-8 -*-
from numpy import pi, linspace, zeros, ones, dot, squeeze
from SciDataTool import Data1D, DataTime
from ....Functions.Electrical.coordinate_transformation import dq2n
from ....Functions.Winding.gen_phase_list import gen_name
from pyleecan.Classes.Winding import Winding


def comp_mmf_unit(self, Na=None, Nt=None, freq=1):
    """Compute the winding Unit magnetomotive force

    Parameters
    ----------
    self : LamSlotWind
        an LamSlotWind object
    Na : int
        Space discretization for offline computation (otherwise use out.elec.angle)
    Nt : int
        Time discretization for offline computation (otherwise use out.elec.time)
    freq : float
        Stator current frequency to consider

    Returns
    -------
    MMF_U : SciDataTool.Classes.DataND.DataND
        Unit magnetomotive force (Na,Nt)
    WF : SciDataTool.Classes.DataND.DataND
        Winding functions (qs,Na)

    """

    # Get stator winding number of phases
    qs = self.winding.qs

    # Get spatial symmetry
    per_a, _, _, _ = self.comp_periodicity()

    # Define the space dicretization
    angle = linspace(0, 2 * pi / per_a, Na, endpoint=False)

    # Define the time dicretization
    time = linspace(0, 1 / freq, Nt, endpoint=False)

    # Compute the winding function and mmf
    if self.winding is None or type(self.winding) is Winding:
        wf = zeros((qs, Na))
    else:
        wf = self.comp_wind_function(angle=angle, per_a=per_a)

    # Compute unit current function of time applying constant Id=1 Arms, Iq=0
    Idq = zeros((Nt, 2))
    Idq[:, 0] = ones(Nt)
    I = dq2n(Idq, 2 * pi * freq * time, n=qs, is_n_rms=False)

    # Compute unit mmf
    mmf_u = squeeze(dot(I, wf))

    # Create a Data object
    Time = Data1D(name="time", unit="s", values=time)
    Angle = Data1D(
        name="angle",
        unit="rad",
        symmetries={"period": per_a},
        values=angle,
        normalizations={"space_order": self.get_pole_pair_number()},
    )
    Phase = Data1D(
        name="phase",
        unit="",
        values=gen_name(qs),
        is_components=True,
    )
    MMF_U = DataTime(
        name="Unit MMF",
        unit="p.u.",
        symbol="Magnitude",
        axes=[Time, Angle],
        values=mmf_u,
    )

    WF = DataTime(
        name="Winding Functions",
        unit="p.u.",
        symbol="Magnitude",
        axes=[Phase, Angle],
        values=wf,
    )

    return MMF_U, WF



def has_magnet(self):
    """Return if the Hole has magnets

    Parameters
    ----------
    self : HoleUD
        A HoleUD object

    Returns
    -------
    has_magnet : bool
        True if the magnets are not None
    """

    has_mag = False
    if self.magnet_dict is None:
        self.magnet_dict = dict()
        return False
    for value in self.magnet_dict.values():
        if value is not None:
            has_mag = True
    return has_mag



# -*- coding: utf-8 -*-

import numpy as np


def add_cell(self, pt_indice, new_ind):
    """Add a new element

    Parameters
    ----------
    self : CellMat
        an CellMat object
    new_ind : int
        an new cell indices
    pt_indice : ndarray
        connectivity

    Returns
    -------
        is_created : bool
            False if the element already exist or if it is not possible to add the element
    """
    # Check the existence of the element
    if len(np.unique(pt_indice)) != self.nb_pt_per_cell:
        return False

    if self.is_exist(pt_indice):
        return False

    # Create the new element
    if self.nb_cell == 0:
        self.connectivity = pt_indice
        self.indice = np.array([new_ind])
    else:
        self.connectivity = np.vstack([self.connectivity, pt_indice])
        self.indice = np.concatenate([self.indice, np.array([new_ind])])

    self.nb_cell = self.nb_cell + 1

    return True



# -*- coding: utf-8 -*-

from ....Methods.Simulation.Input import InputError


def run(self):
    """Run the Force module"""
    if self.parent is None:
        raise InputError(
            "ERROR: The Force object must be in a Simulation object to run"
        )
    if self.parent.parent is None:
        raise InputError("ERROR: The Force object must be in an Output object to run")

    self.get_logger().info("Starting Force module")

    output = self.parent.parent

    # Compute and store time and angle axes from previous output
    # and returns additional axes in axes_dict
    axes_dict = self.comp_axes(output)

    # Compute the magnetic force according to the Force model
    out_dict = self.comp_force(output, axes_dict)

    # Store force quantities contained in out_dict in OutForce, as Data object if necessary
    output.force.store(out_dict, axes_dict)

    # Compute the air-gap surface force transfer if required
    if self.is_agsf_transfer:
        self.comp_AGSF_transfer(output, self.max_wavenumber_transfer)



# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
from ...definitions import config_dict
from ...Functions.Plot.plot_3D_Data import plot_3D_Data
from ...Functions.Plot.plot_2D_Data import plot_2D_Data

FONT_NAME = config_dict["PLOT"]["FONT_NAME"]


def plot_A_time_space(
    data,
    is_deg=True,
    is_elecorder=False,
    is_spaceorder=False,
    freq_max=20000,
    r_max=100,
    z_max=None,
    is_norm=False,
    unit="SI",
    save_path=None,
    is_auto_ticks=True,
    is_show_fig=None,
    fig=None,
    color_list=[],
):
    """Plots a field as a function of time and space (angle)

    Parameters
    ----------
    data : Data
        a Data object
    is_deg : bool
        boolean indicating if the angle must be converted to degrees
    is_elecorder : bool
        boolean indicating if we want to use the electrical order for the fft axis
    is_spaceorder : bool
        boolean indicating if we want to use the spatial order for the fft axis
    freq_max : float
        maximum value of the frequency for the fft axis
    r_max : int
        maximum value of the wavenumber for the fft axis
    z_max : float
        maximum value for the field
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    is_auto_ticks : bool
        in fft, adjust ticks to freqs and wavenumbers (deactivate if too close)
    is_show_fig : bool
        True to show figure after plot
    color_list : list
        list of colors to use for each curve
    """

    if is_show_fig is None:
        is_show_fig = True if fig is None else False

    # Set plot
    fig, axs = plt.subplots(3, 2, tight_layout=True, figsize=(20, 10))
    title = data.name + " over time and space"

    # pcolorplot
    if is_deg:
        angle_str = "angle{°}"
    else:
        angle_str = "angle{rad}"

    plot_3D_Data(
        data,
        "time",
        angle_str,
        is_norm=is_norm,
        unit=unit,
        z_max=z_max,
        fig=fig,
        ax=axs[0, 0],
        is_auto_ticks=is_auto_ticks,
        is_show_fig=False,
        is_2D_view=True,
    )

    # 2D plots
    # time
    plot_2D_Data(
        data,
        "time",
        fig=fig,
        ax=axs[1, 0],
        color_list=color_list,
        is_auto_ticks=is_auto_ticks,
        is_show_fig=False,
    )

    # angle
    plot_2D_Data(
        data,
        angle_str,
        fig=fig,
        ax=axs[2, 0],
        color_list=color_list,
        is_auto_ticks=is_auto_ticks,
        is_show_fig=False,
    )

    # fft time
    if is_elecorder:
        elec_max = None
        for ax in data.axes:
            if ax.name == "time":
                try:
                    elec_max = freq_max / ax.normalizations["elec_order"]
                except:
                    pass

        if elec_max is None:
            freq_str = "freqs=[0," + str(freq_max) + "]"
        else:
            freq_str = "freqs->elec_order[0," + str(elec_max) + "]"
    else:
        freq_str = "freqs=[0," + str(freq_max) + "]"

    plot_2D_Data(
        data,
        freq_str,
        fig=fig,
        ax=axs[1, 1],
        unit=unit,
        color_list=color_list,
        is_auto_ticks=is_auto_ticks,
        is_show_fig=False,
    )

    # fft space
    if is_spaceorder:
        order_max = None
        for ax in data.axes:
            if ax.name == "angle":
                try:
                    order_max = r_max / ax.normalizations["space_order"]
                except:
                    pass

        if order_max is None:
            wavenb_str = "wavenumber=[0," + str(r_max) + "]"
        else:
            wavenb_str = "wavenumber->space_order[0," + str(order_max) + "]"
    else:
        wavenb_str = "wavenumber=[0," + str(r_max) + "]"

    plot_2D_Data(
        data,
        wavenb_str,
        fig=fig,
        ax=axs[2, 1],
        unit=unit,
        color_list=color_list,
        is_auto_ticks=is_auto_ticks,
        is_show_fig=False,
    )

    axs[0, 1].axis("off")
    axs[0, 1].set_title("")

    fig.canvas.set_window_title(title)
    fig.suptitle(title, x=0.65, fontsize=24, fontname=FONT_NAME)
    fig.tight_layout()

    if save_path is not None:
        fig.savefig(save_path)
        plt.close()

    if is_show_fig:
        fig.show()



from ....Methods.Machine.Lamination import LaminationCheckError


class Lam_SlotCheckError(LaminationCheckError):
    """ """

    pass


class LSC_OutSlotInLam(Lam_SlotCheckError):
    """ """

    pass


class LSC_InSlotOutLam(Lam_SlotCheckError):
    """ """

    pass


class LSC_SlotTooLong(Lam_SlotCheckError):
    """ """

    pass


class LSC_OverlappingSlot(Lam_SlotCheckError):
    """ """

    pass



from ....Methods.Slot.Slot import SlotCheckError


class S52_NoneError(SlotCheckError):
    """Raised when a propery of HoleM52 is None"""

    pass


class S52_H12CheckError(SlotCheckError):
    """ """

    pass


class S52_alphaCheckError(SlotCheckError):
    """ """

    pass


class S52_W1CheckError(SlotCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-
"""File generated according to PMSlot14/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SMSlot.PMSlot14.Ui_PMSlot14 import Ui_PMSlot14


class Gen_PMSlot14(Ui_PMSlot14):
    def setupUi(self, PMSlot14):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PMSlot14.setupUi(self, PMSlot14)
        # Setup of in_W0
        txt = self.tr(u"""Slot isthmus angular width.""")
        self.in_W0.setWhatsThis(txt)
        self.in_W0.setToolTip(txt)

        # Setup of lf_W0
        self.lf_W0.validator().setBottom(0)
        txt = self.tr(u"""Slot isthmus angular width.""")
        self.lf_W0.setWhatsThis(txt)
        self.lf_W0.setToolTip(txt)

        # Setup of in_Wmag
        txt = self.tr(u"""Magnet angular width""")
        self.in_Wmag.setWhatsThis(txt)
        self.in_Wmag.setToolTip(txt)

        # Setup of lf_Wmag
        self.lf_Wmag.validator().setBottom(0)
        txt = self.tr(u"""Magnet angular width""")
        self.lf_Wmag.setWhatsThis(txt)
        self.lf_Wmag.setToolTip(txt)

        # Setup of in_H0
        txt = self.tr(u"""Slot isthmus height.""")
        self.in_H0.setWhatsThis(txt)
        self.in_H0.setToolTip(txt)

        # Setup of lf_H0
        self.lf_H0.validator().setBottom(0)
        txt = self.tr(u"""Slot isthmus height.""")
        self.lf_H0.setWhatsThis(txt)
        self.lf_H0.setToolTip(txt)

        # Setup of in_Hmag
        txt = self.tr(u"""Magnet Height""")
        self.in_Hmag.setWhatsThis(txt)
        self.in_Hmag.setToolTip(txt)

        # Setup of lf_Hmag
        self.lf_Hmag.validator().setBottom(0)
        txt = self.tr(u"""Magnet Height""")
        self.lf_Hmag.setWhatsThis(txt)
        self.lf_Hmag.setToolTip(txt)



# -*- coding: utf-8 -*-
from ....Methods.Elmer.Section import ElmerSectionError


def pop(self, key):
    """
    Remove the item with the given key and return it.
    """
    if key not in self.keys():
        raise ElmerSectionError("Key doesn't exist.")

    value = self._statements.pop(key)
    comment = self._comments.pop(key)

    return (value, comment)



# -*- coding: utf-8 -*-


def comp_Rgap_mec(self):
    """Returns the radius of the center of the mecanical airgap

    Parameters
    ----------
    self : Machine
        Machine object

    Returns
    -------
    Rgap_mec: float
        Radius of the center of the mecanical airgap [m]

    """
    stator = self.stator
    rotor = self.rotor
    return (stator.comp_radius_mec() + rotor.comp_radius_mec()) / 2



from ....Functions.Geometry.merge_notch_list import merge_notch_list
from ....Classes.Arc1 import Arc1
from numpy import exp, pi


def get_bore_desc(self, sym=1):
    """This method returns an ordered description of the elements
    that defines the bore radius of the lamination

    Parameters
    ----------
    self : LamSlot
        A LamSlot object

    Returns
    -------
    bore_desc : list
        list of dictionary with key: "begin_angle", "end_angle", "obj"
    """

    Rbo = self.get_Rbo()
    slot_list = list()
    slot = self.slot
    Zs = self.slot.Zs
    op = slot.comp_angle_opening()
    # To avoid calling build_geometry for all the slot
    lines = slot.build_geometry()
    # First add all the slots
    for ii in range(Zs // sym):
        bore_dict = dict()
        bore_dict["begin_angle"] = 2 * pi / Zs * ii - op / 2 + pi / Zs
        bore_dict["end_angle"] = 2 * pi / Zs * ii + op / 2 + pi / Zs
        bore_dict["obj"] = slot
        bore_dict["lines"] = lines
        slot_list.append(bore_dict)

    # Get the notches
    notch_list = self.get_notch_list(sym=sym)

    # Merge Slot and Notches
    merged_list = merge_notch_list(slot_list, notch_list)

    # Add all the bore lines
    bore_desc = list()
    for ii, desc in enumerate(merged_list):
        bore_desc.append(desc)
        if ii != len(merged_list) - 1 and abs(op - 2 * pi / Zs) > 1e-6:
            bore_dict = dict()
            bore_dict["begin_angle"] = merged_list[ii]["end_angle"]
            bore_dict["end_angle"] = merged_list[ii + 1]["begin_angle"]
            bore_dict["obj"] = Arc1(
                begin=Rbo * exp(1j * bore_dict["begin_angle"]),
                end=Rbo * exp(1j * bore_dict["end_angle"]),
                radius=Rbo,
                is_trigo_direction=True,
            )
            bore_desc.append(bore_dict)

    # Add last bore line
    if sym == 1 and abs(op - 2 * pi / Zs) > 1e-6:
        bore_dict = dict()
        bore_dict["begin_angle"] = merged_list[-1]["end_angle"]
        bore_dict["end_angle"] = merged_list[0]["begin_angle"]
        bore_dict["obj"] = Arc1(
            begin=Rbo * exp(1j * bore_dict["begin_angle"]),
            end=Rbo * exp(1j * bore_dict["end_angle"]),
            radius=Rbo,
            is_trigo_direction=True,
        )
        if merged_list[0]["begin_angle"] < 0:
            # First element is an slot or notch
            bore_desc.append(bore_dict)
        else:
            # First element is a bore line
            bore_desc.insert(0, bore_dict)
    elif sym != 1 and abs(op - 2 * pi / Zs) > 1e-6:  # With symmetry
        # Add last bore line
        bore_dict = dict()
        bore_dict["begin_angle"] = merged_list[-1]["end_angle"]
        bore_dict["end_angle"] = 2 * pi / sym
        bore_dict["obj"] = Arc1(
            begin=Rbo * exp(1j * bore_dict["begin_angle"]),
            end=Rbo * exp(1j * bore_dict["end_angle"]),
            radius=Rbo,
            is_trigo_direction=True,
        )
        bore_desc.append(bore_dict)

        # Add first bore line
        bore_dict = dict()
        bore_dict["begin_angle"] = 0
        bore_dict["end_angle"] = merged_list[0]["begin_angle"]
        bore_dict["obj"] = Arc1(
            begin=Rbo * exp(1j * bore_dict["begin_angle"]),
            end=Rbo * exp(1j * bore_dict["end_angle"]),
            radius=Rbo,
            is_trigo_direction=True,
        )
        bore_desc.insert(0, bore_dict)
    return bore_desc



# -*- coding: utf-8 -*-

from ....Methods.Slot.Slot import SlotCheckError
from numpy import pi
from ....Methods.Slot.HoleM51 import *


def check(self):
    """Check that the HoleM51 object is correct

    Parameters
    ----------
    self : HoleM51
        A HoleM51 object

    Returns
    -------
    None
    """

    # Check that everything is set
    if self.W0 is None:
        raise S51_NoneError("You must set W0 !")
    elif self.W1 is None:
        raise S51_NoneError("You must set W1 !")
    elif self.W2 is None:
        raise S51_NoneError("You must set W2 !")
    elif self.W3 is None:
        raise S51_NoneError("You must set W3 !")
    elif self.W4 is None:
        raise S51_NoneError("You must set W4 !")
    elif self.W5 is None:
        raise S51_NoneError("You must set W5 !")
    elif self.W6 is None:
        raise S51_NoneError("You must set W6 !")
    elif self.W7 is None:
        raise S51_NoneError("You must set W7 !")
    elif self.H0 is None:
        raise S51_NoneError("You must set H0 !")
    elif self.H1 is None:
        raise S51_NoneError("You must set H1 !")
    elif self.H2 is None:
        raise S51_NoneError("You must set H2 !")

    Rext = self.get_Rext()
    alpha = self.comp_alpha()

    if self.W0 < (self.W2 + self.W3):
        raise S51_WCheckError("You must have W2+W3 < W0")

    if Rext < self.H0 + self.H2:
        raise S51_RHCheckError("You must have H0+H2 < Rext")

    if alpha > pi / 2:
        raise S51_AlphaCheckError("You must have alpha < pi/2")



from numpy import ones


class OutElecError(Exception):
    pass


def get_Nr(self):
    """Create speed in function of time vector Nr

    Parameters
    ----------
    self : OutElec
        An OutElec object

    Returns
    -------
    Nr: ndarray
        speed in function of time
    """
    if self.Time is None:
        raise OutElecError('You must define "time" property before calling get_Nr')
    if self.N0 is None:
        raise OutElecError('You must define "N0" before calling get_Nr.')

    # Same speed for every timestep
    Nr = self.N0 * ones(self.Time.get_length(is_oneperiod=False))
    return Nr



# -*- coding: utf-8 -*-

from numpy import pi
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QMessageBox, QWidget

from .....Classes.LamSlotWind import LamSlotWind
from .....Classes.Slot import Slot
from .....Classes.SlotM10 import SlotM10
from .....Classes.Slot import Slot
from .....GUI.Dialog.DMachineSetup.SMSlot.Ui_SMSlot import Ui_SMSlot
from .....GUI.Dialog.DMachineSetup.SMSlot.PMSlot10.PMSlot10 import PMSlot10
from .....GUI.Dialog.DMachineSetup.SMSlot.PMSlot11.PMSlot11 import PMSlot11
from .....GUI.Dialog.DMachineSetup.SMSlot.PMSlot12.PMSlot12 import PMSlot12
from .....GUI.Dialog.DMachineSetup.SMSlot.PMSlot13.PMSlot13 import PMSlot13
from .....GUI.Dialog.DMachineSetup.SMSlot.PMSlot14.PMSlot14 import PMSlot14
from .....GUI.Dialog.DMachineSetup.SMSlot.PMSlot15.PMSlot15 import PMSlot15
from .....GUI.Dialog.DMachineSetup.SMSlot.PMSlot16.PMSlot16 import PMSlot16
from .....GUI.Dialog.DMachineSetup.SMSlot.PMSlot17.PMSlot17 import PMSlot17

# List to convert index of combobox to slot type
WIDGET_LIST = [
    PMSlot10,
    PMSlot11,
    PMSlot12,
    PMSlot13,
    PMSlot14,
    PMSlot15,
    PMSlot16,
    PMSlot17,
]
INIT_INDEX = [wid.slot_type for wid in WIDGET_LIST]
SLOT_NAME = [wid.slot_name for wid in WIDGET_LIST]


class SMSlot(Ui_SMSlot, QWidget):
    """Step to set the slot with winding"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for DMachineSetup nav
    step_name = "Magnet"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the GUI according to machine

        Parameters
        ----------
        self : SMSlot
            A SMSlot widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        self.b_help.hide()

        # Fill the combobox with the available slot
        self.c_slot_type.clear()
        for slot in SLOT_NAME:
            self.c_slot_type.addItem(slot)
        # Avoid erase all the parameters when navigating though the slots
        self.previous_slot = dict()
        for slot_type in INIT_INDEX:
            self.previous_slot[slot_type] = None

        if self.is_stator:
            self.obj = machine.stator
        else:
            self.obj = machine.rotor

        # If the Slot is not set, initialize it with a SlotM10
        if self.obj.slot is None or type(self.obj.slot) is Slot:
            self.obj.slot = SlotM10()
            self.obj.slot._set_None()

        self.set_slot_pitch(self.obj.slot.Zs)

        # Set magnetization
        if self.obj.magnet.type_magnetization not in [0, 1, 2]:
            self.obj.magnet.type_magnetization = 0  # Set default value
        self.c_type_magnetization.setCurrentIndex(self.obj.magnet.type_magnetization)

        # Set material
        self.w_mat.setText(self.tr("mat_mag:"))
        self.w_mat.def_mat = "Magnet1"
        self.w_mat.update(self.machine.rotor.magnet, "mat_type", self.matlib)

        # Set the correct index for the type checkbox and display the object
        index = INIT_INDEX.index(type(self.obj.slot))
        self.c_slot_type.setCurrentIndex(index)

        # Update the slot widget
        self.s_update_slot()

        # Connect the slot
        self.c_slot_type.currentIndexChanged.connect(self.s_change_slot)
        self.c_type_magnetization.currentIndexChanged.connect(
            self.s_set_type_magnetization
        )
        self.b_plot.clicked.connect(self.s_plot)
        self.w_mat.saveNeeded.connect(self.emit_save)

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()

    def set_slot_type(self, index):
        """Initialize self.obj with the slot corresponding to index

        Parameters
        ----------
        self : SMSlot
            A SMSlot object
        index : int
            Index of the selected slot type in the list
        """

        # Save the slot
        self.previous_slot[type(self.obj.slot)] = self.obj.slot

        # Call the corresponding constructor
        Zs = self.obj.slot.Zs
        if self.previous_slot[INIT_INDEX[index]] is None:
            # No previous slot of this type
            self.obj.slot = INIT_INDEX[index]()
            self.obj.slot._set_None()  # No default value
            self.obj.slot.Zs = Zs
        else:  # Load the previous slot of this type
            self.obj.slot = self.previous_slot[INIT_INDEX[index]]
            if self.obj.slot.Zs is not None:
                self.set_slot_pitch(self.obj.slot.Zs)

        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_slot_pitch(self, Zs):
        """Update out_slot_pitch with the correct value

        Parameters
        ----------
        self : SMSlot
            A SMSlot object
        Zs : int
            The current value of Zs
        """
        sp_txt = self.tr("p = " + str(int(Zs / 2)) + " / Slot pitch = ")

        Slot_pitch = 360.0 / Zs
        Slot_pitch_rad = Slot_pitch * pi / 180

        self.out_Slot_pitch.setText(
            sp_txt
            + "%.4g" % (Slot_pitch)
            + u" [°] ("
            + "%.4g" % (Slot_pitch_rad)
            + " [rad])"
        )

    def s_set_type_magnetization(self, index):
        """Signal to update the value of type_magnetization according to the combobox

        Parameters
        ----------
        self : SMagnet
            A SMagnet object
        index : int
            Current index of the combobox
        """
        self.machine.rotor.magnet.type_magnetization = index
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def s_update_slot(self):
        """Update the slot widget

        Parameters
        ----------
        self : SMSlot
            A SMSlot object
        """

        # Regenerate the pages with the new values
        self.w_slot.setParent(None)
        self.w_slot = WIDGET_LIST[self.c_slot_type.currentIndex()](self.obj)
        self.w_slot.saveNeeded.connect(self.emit_save)
        # Refresh the GUI
        self.main_layout.removeWidget(self.w_slot)
        self.main_layout.insertWidget(2, self.w_slot)

    def s_change_slot(self, index):
        """Signal to update the slot object and widget

        Parameters
        ----------
        self : SMSlot
            A SMSlot object
        index : int
            Current index of the combobox
        """
        # Current slot is removed and replaced by the new one
        self.set_slot_type(index)
        self.s_update_slot()

    def s_plot(self):
        """Try to plot the lamination

        Parameters
        ----------
        self : SMSlot
            A SMSlot object
        """
        # We have to make sure the slot is right before trying to plot it
        error = self.check(self.obj)

        if error:  # Error => Display it
            QMessageBox().critical(self, self.tr("Error"), error)
        else:  # No error => Plot the slot (No winding for LamSquirrelCage)
            self.obj.plot()

    @staticmethod
    def check(lam):
        """Check that the current lamination have all the needed field set

        Parameters
        ----------
        lam: LamSlotMag
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """
        try:
            # Call the check method of the slot (every slot type have a
            # different check method)
            index = INIT_INDEX.index(type(lam.slot))
            return WIDGET_LIST[index].check(lam)
        except Exception as e:
            return str(e)



# -*- coding: utf-8 -*-

from matplotlib.pyplot import axis, legend
from numpy import real, imag

from ....Functions.init_fig import init_fig


def plot_lines(self, fig=None, is_show_fig=True):
    """Plot the SurfLine-Contour in a matplotlib fig
    (For plotting unclosed contour, for Polygon use plot method from Surface object)

    Parameters
    ----------
    self : SurfLine
        A SurfLine object
    fig :
        if None, open a new fig and plot, else add to the
        current one (Default value = None)
    is_show_fig : bool
        To call show at the end of the method

    Returns
    -------
    None
    """

    (fig, axes, patch_leg, label_leg) = init_fig(fig)
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")

    points = self.discretize(10)

    for idx in range(len(points) - 1):
        z1 = points[idx]
        z2 = points[idx + 1]
        x1 = real(z1)
        y1 = imag(z1)
        x2 = real(z2)
        y2 = imag(z2)
        axes.plot([x1, x2], [y1, y2], "k")

    # Axis Setup
    axis("equal")

    if is_show_fig:
        fig.show()



def comp_desc_dict(self):
    """Compute a dictionnary with the main parameters/output of the machine

    Parameters
    ----------
    self : Machine
        A Machine object

    Returns
    -------
    desc_dict: list
        list of dictionnary containing the main parameters of the machine
    """

    desc_dict = list()

    # Machine Type
    desc_str = self.get_machine_type()
    desc_dict.append(
        dict(
            {
                "name": "Type",
                "path": "type(machine)",
                "verbose": "Machine Type",
                "type": str,
                "unit": "",
                "is_input": False,
                "value": desc_str.split(" ")[0],
            }
        )
    )
    # Zs
    desc_dict.append(
        dict(
            {
                "name": "Zs",
                "path": "machine.stator.slot.Zs",
                "verbose": "Stator slot number",
                "type": int,
                "unit": "",
                "is_input": True,
                "value": self.stator.slot.Zs,
            }
        )
    )
    # p
    desc_dict.append(
        dict(
            {
                "name": "p",
                "path": "machine.stator.winding.p",
                "verbose": "Pole pair number",
                "type": int,
                "unit": "",
                "is_input": True,
                "value": self.stator.get_pole_pair_number(),
            }
        )
    )
    # is_inner_rotor
    if self.rotor.is_internal:
        inner = "Inner Rotor"
    else:
        inner = "Outer Rotor"
    desc_dict.append(
        dict(
            {
                "name": "Topology",
                "path": "machine.rotor.is_internal",
                "verbose": "Topology",
                "type": str,
                "unit": "",
                "is_input": False,
                "value": inner,
            }
        )
    )
    # qs
    desc_dict.append(
        dict(
            {
                "name": "qs",
                "path": "machine.stator.winding.qs",
                "verbose": "Stator phase number",
                "type": int,
                "unit": "",
                "is_input": True,
                "value": self.stator.winding.qs,
            }
        )
    )
    # Stator winding type
    desc_dict.append(
        dict(
            {
                "name": "Winding",
                "path": "machine.stator.winding.NAME",
                "verbose": "Stator winding type",
                "type": str,
                "unit": "",
                "is_input": False,
                "value": self.stator.winding.NAME,
            }
        )
    )
    # Stator winding resistance
    try:
        Rwind = self.stator.comp_resistance_wind()
    except Exception:
        Rwind = None
    desc_dict.append(
        dict(
            {
                "name": "Rwinds",
                "path": "machine.stator.comp_resistance_wind()",
                "verbose": "Stator winding resistance",
                "type": float,
                "unit": "Ohm",
                "is_input": False,
                "value": Rwind,
            }
        )
    )
    # Machine mass
    try:
        Mmach = self.comp_masses()["Mmach"]
    except Exception:
        Mmach = None
    desc_dict.append(
        dict(
            {
                "name": "Mmachine",
                "path": "machine.comp_masses()['Mmach']",
                "verbose": "Machine total mass",
                "type": float,
                "unit": "kg",
                "is_input": False,
                "value": Mmach,
            }
        )
    )

    return desc_dict



# -*- coding: utf-8 -*-


def get_Rbo(self):
    """Return the bore lamination radius

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    Rbo: float
        The lamination bore radius [m]

    """

    if self.is_internal:
        return self.Rext
    else:
        return self.Rint



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/InputElec.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/InputElec
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Input import Input

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.InputElec.gen_input import gen_input
except ImportError as error:
    gen_input = error

try:
    from ..Methods.Simulation.InputElec.comp_felec import comp_felec
except ImportError as error:
    comp_felec = error

try:
    from ..Methods.Simulation.InputElec.set_Id_Iq import set_Id_Iq
except ImportError as error:
    set_Id_Iq = error


from ..Classes.ImportMatrixVal import ImportMatrixVal
from numpy import ndarray
from numpy import array, array_equal
from ._check import InitUnKnowClassError
from .ImportMatrix import ImportMatrix


class InputElec(Input):
    """Input to skip the electrical module and start with the magnetic one"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.InputElec.gen_input
    if isinstance(gen_input, ImportError):
        gen_input = property(
            fget=lambda x: raise_(
                ImportError("Can't use InputElec method gen_input: " + str(gen_input))
            )
        )
    else:
        gen_input = gen_input
    # cf Methods.Simulation.InputElec.comp_felec
    if isinstance(comp_felec, ImportError):
        comp_felec = property(
            fget=lambda x: raise_(
                ImportError("Can't use InputElec method comp_felec: " + str(comp_felec))
            )
        )
    else:
        comp_felec = comp_felec
    # cf Methods.Simulation.InputElec.set_Id_Iq
    if isinstance(set_Id_Iq, ImportError):
        set_Id_Iq = property(
            fget=lambda x: raise_(
                ImportError("Can't use InputElec method set_Id_Iq: " + str(set_Id_Iq))
            )
        )
    else:
        set_Id_Iq = set_Id_Iq
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        rot_dir=-1,
        Id_ref=None,
        Iq_ref=None,
        Ud_ref=None,
        Uq_ref=None,
        felec=None,
        time=None,
        angle=None,
        Nt_tot=2048,
        Nrev=1,
        Na_tot=2048,
        N0=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "rot_dir" in list(init_dict.keys()):
                rot_dir = init_dict["rot_dir"]
            if "Id_ref" in list(init_dict.keys()):
                Id_ref = init_dict["Id_ref"]
            if "Iq_ref" in list(init_dict.keys()):
                Iq_ref = init_dict["Iq_ref"]
            if "Ud_ref" in list(init_dict.keys()):
                Ud_ref = init_dict["Ud_ref"]
            if "Uq_ref" in list(init_dict.keys()):
                Uq_ref = init_dict["Uq_ref"]
            if "felec" in list(init_dict.keys()):
                felec = init_dict["felec"]
            if "time" in list(init_dict.keys()):
                time = init_dict["time"]
            if "angle" in list(init_dict.keys()):
                angle = init_dict["angle"]
            if "Nt_tot" in list(init_dict.keys()):
                Nt_tot = init_dict["Nt_tot"]
            if "Nrev" in list(init_dict.keys()):
                Nrev = init_dict["Nrev"]
            if "Na_tot" in list(init_dict.keys()):
                Na_tot = init_dict["Na_tot"]
            if "N0" in list(init_dict.keys()):
                N0 = init_dict["N0"]
        # Set the properties (value check and convertion are done in setter)
        self.rot_dir = rot_dir
        self.Id_ref = Id_ref
        self.Iq_ref = Iq_ref
        self.Ud_ref = Ud_ref
        self.Uq_ref = Uq_ref
        self.felec = felec
        # Call Input init
        super(InputElec, self).__init__(
            time=time, angle=angle, Nt_tot=Nt_tot, Nrev=Nrev, Na_tot=Na_tot, N0=N0
        )
        # The class is frozen (in Input init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        InputElec_str = ""
        # Get the properties inherited from Input
        InputElec_str += super(InputElec, self).__str__()
        InputElec_str += "rot_dir = " + str(self.rot_dir) + linesep
        InputElec_str += "Id_ref = " + str(self.Id_ref) + linesep
        InputElec_str += "Iq_ref = " + str(self.Iq_ref) + linesep
        InputElec_str += "Ud_ref = " + str(self.Ud_ref) + linesep
        InputElec_str += "Uq_ref = " + str(self.Uq_ref) + linesep
        InputElec_str += "felec = " + str(self.felec) + linesep
        return InputElec_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Input
        if not super(InputElec, self).__eq__(other):
            return False
        if other.rot_dir != self.rot_dir:
            return False
        if other.Id_ref != self.Id_ref:
            return False
        if other.Iq_ref != self.Iq_ref:
            return False
        if other.Ud_ref != self.Ud_ref:
            return False
        if other.Uq_ref != self.Uq_ref:
            return False
        if other.felec != self.felec:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Input
        diff_list.extend(super(InputElec, self).compare(other, name=name))
        if other._rot_dir != self._rot_dir:
            diff_list.append(name + ".rot_dir")
        if other._Id_ref != self._Id_ref:
            diff_list.append(name + ".Id_ref")
        if other._Iq_ref != self._Iq_ref:
            diff_list.append(name + ".Iq_ref")
        if other._Ud_ref != self._Ud_ref:
            diff_list.append(name + ".Ud_ref")
        if other._Uq_ref != self._Uq_ref:
            diff_list.append(name + ".Uq_ref")
        if other._felec != self._felec:
            diff_list.append(name + ".felec")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Input
        S += super(InputElec, self).__sizeof__()
        S += getsizeof(self.rot_dir)
        S += getsizeof(self.Id_ref)
        S += getsizeof(self.Iq_ref)
        S += getsizeof(self.Ud_ref)
        S += getsizeof(self.Uq_ref)
        S += getsizeof(self.felec)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Input
        InputElec_dict = super(InputElec, self).as_dict(**kwargs)
        InputElec_dict["rot_dir"] = self.rot_dir
        InputElec_dict["Id_ref"] = self.Id_ref
        InputElec_dict["Iq_ref"] = self.Iq_ref
        InputElec_dict["Ud_ref"] = self.Ud_ref
        InputElec_dict["Uq_ref"] = self.Uq_ref
        InputElec_dict["felec"] = self.felec
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        InputElec_dict["__class__"] = "InputElec"
        return InputElec_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.rot_dir = None
        self.Id_ref = None
        self.Iq_ref = None
        self.Ud_ref = None
        self.Uq_ref = None
        self.felec = None
        # Set to None the properties inherited from Input
        super(InputElec, self)._set_None()

    def _get_rot_dir(self):
        """getter of rot_dir"""
        return self._rot_dir

    def _set_rot_dir(self, value):
        """setter of rot_dir"""
        check_var("rot_dir", value, "float", Vmin=-1, Vmax=1)
        self._rot_dir = value

    rot_dir = property(
        fget=_get_rot_dir,
        fset=_set_rot_dir,
        doc=u"""Rotation direction of the rotor 1 trigo, -1 clockwise

        :Type: float
        :min: -1
        :max: 1
        """,
    )

    def _get_Id_ref(self):
        """getter of Id_ref"""
        return self._Id_ref

    def _set_Id_ref(self, value):
        """setter of Id_ref"""
        check_var("Id_ref", value, "float")
        self._Id_ref = value

    Id_ref = property(
        fget=_get_Id_ref,
        fset=_set_Id_ref,
        doc=u"""d-axis current magnitude

        :Type: float
        """,
    )

    def _get_Iq_ref(self):
        """getter of Iq_ref"""
        return self._Iq_ref

    def _set_Iq_ref(self, value):
        """setter of Iq_ref"""
        check_var("Iq_ref", value, "float")
        self._Iq_ref = value

    Iq_ref = property(
        fget=_get_Iq_ref,
        fset=_set_Iq_ref,
        doc=u"""q-axis current magnitude

        :Type: float
        """,
    )

    def _get_Ud_ref(self):
        """getter of Ud_ref"""
        return self._Ud_ref

    def _set_Ud_ref(self, value):
        """setter of Ud_ref"""
        check_var("Ud_ref", value, "float")
        self._Ud_ref = value

    Ud_ref = property(
        fget=_get_Ud_ref,
        fset=_set_Ud_ref,
        doc=u"""d-axis voltage magnitude

        :Type: float
        """,
    )

    def _get_Uq_ref(self):
        """getter of Uq_ref"""
        return self._Uq_ref

    def _set_Uq_ref(self, value):
        """setter of Uq_ref"""
        check_var("Uq_ref", value, "float")
        self._Uq_ref = value

    Uq_ref = property(
        fget=_get_Uq_ref,
        fset=_set_Uq_ref,
        doc=u"""q-axis voltage magnitude

        :Type: float
        """,
    )

    def _get_felec(self):
        """getter of felec"""
        return self._felec

    def _set_felec(self, value):
        """setter of felec"""
        check_var("felec", value, "float")
        self._felec = value

    felec = property(
        fget=_get_felec,
        fset=_set_felec,
        doc=u"""electrical frequency

        :Type: float
        """,
    )



# -*- coding: utf-8 -*-

# File generated according to SWPole.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Tools.HelpButton import HelpButton

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SWPole(object):
    def setupUi(self, SWPole):
        if not SWPole.objectName():
            SWPole.setObjectName(u"SWPole")
        SWPole.resize(650, 550)
        SWPole.setMinimumSize(QSize(650, 550))
        self.main_layout = QVBoxLayout(SWPole)
        self.main_layout.setObjectName(u"main_layout")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.b_help = HelpButton(SWPole)
        self.b_help.setObjectName(u"b_help")
        self.b_help.setPixmap(QPixmap(u":/images/images/icon/help_16.png"))

        self.horizontalLayout_2.addWidget(self.b_help)

        self.c_slot_type = QComboBox(SWPole)
        self.c_slot_type.addItem("")
        self.c_slot_type.addItem("")
        self.c_slot_type.setObjectName(u"c_slot_type")

        self.horizontalLayout_2.addWidget(self.c_slot_type)

        self.in_Zs = QLabel(SWPole)
        self.in_Zs.setObjectName(u"in_Zs")

        self.horizontalLayout_2.addWidget(self.in_Zs)

        self.out_Slot_pitch = QLabel(SWPole)
        self.out_Slot_pitch.setObjectName(u"out_Slot_pitch")

        self.horizontalLayout_2.addWidget(self.out_Slot_pitch)

        self.horizontalSpacer_3 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer_3)

        self.main_layout.addLayout(self.horizontalLayout_2)

        self.w_slot = QWidget(SWPole)
        self.w_slot.setObjectName(u"w_slot")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.w_slot.sizePolicy().hasHeightForWidth())
        self.w_slot.setSizePolicy(sizePolicy)

        self.main_layout.addWidget(self.w_slot)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.b_plot = QPushButton(SWPole)
        self.b_plot.setObjectName(u"b_plot")

        self.horizontalLayout.addWidget(self.b_plot)

        self.b_previous = QPushButton(SWPole)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout.addWidget(self.b_previous)

        self.b_next = QPushButton(SWPole)
        self.b_next.setObjectName(u"b_next")

        self.horizontalLayout.addWidget(self.b_next)

        self.main_layout.addLayout(self.horizontalLayout)

        self.retranslateUi(SWPole)

        QMetaObject.connectSlotsByName(SWPole)

    # setupUi

    def retranslateUi(self, SWPole):
        SWPole.setWindowTitle(QCoreApplication.translate("SWPole", u"Form", None))
        self.b_help.setText("")
        self.c_slot_type.setItemText(
            0, QCoreApplication.translate("SWPole", u"Pole Type 60", None)
        )
        self.c_slot_type.setItemText(
            1, QCoreApplication.translate("SWPole", u"Pole Type 61", None)
        )

        self.in_Zs.setText(QCoreApplication.translate("SWPole", u"Zs = 2*p = ", None))
        self.out_Slot_pitch.setText(
            QCoreApplication.translate("SWPole", u"Slot pitch = 2*Pi / Zs = ", None)
        )
        self.b_plot.setText(QCoreApplication.translate("SWPole", u"Preview", None))
        self.b_previous.setText(QCoreApplication.translate("SWPole", u"Previous", None))
        self.b_next.setText(QCoreApplication.translate("SWPole", u"Next", None))

    # retranslateUi



# -*- coding: utf-8 -*-

from numpy import pi

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.HoleM53 import *


def check(self):
    """Check that the HoleM53 object is correct

    Parameters
    ----------
    self : HoleM53
        A HoleM53 object

    Returns
    -------
    None

    Raises
    -------
    S53_Rext0CheckError
        You must have H0 < Rext
    S53_Rext1CheckError
        You must have H1 < Rext
    S53_W4CheckError
        You must have W4 < pi/2
    S53_W5CheckError
        You must have W5 >=0
    """

    # Check that everything is set
    if self.W1 is None:
        raise S53_NoneError("You must set W1 !")
    elif self.W2 is None:
        raise S53_NoneError("You must set W2 !")
    elif self.W3 is None:
        raise S53_NoneError("You must set W3 !")
    elif self.W4 is None:
        raise S53_NoneError("You must set W4 !")
    elif self.H0 is None:
        raise S53_NoneError("You must set H0 !")
    elif self.H1 is None:
        raise S53_NoneError("You must set H1 !")
    elif self.H2 is None:
        raise S53_NoneError("You must set H2 !")
    elif self.H3 is None:
        raise S53_NoneError("You must set H3 !")

    Rext = self.get_Rext()

    if Rext <= self.H0:
        raise S53_Rext0CheckError("You must have H0 < Rext")

    if Rext <= self.H1:
        raise S53_Rext1CheckError("You must have H1 < Rext")

    if pi / 2 <= self.W4:
        raise S53_W4CheckError("You must have W4 < pi/2")

    if self.comp_W5() < 0:
        raise S53_W5CheckError("You must have W5 >=0")



# -*- coding: utf-8 -*-

import numpy as np


def get_normal(self, vertice):
    """Return the normal vector.

    Parameters
    ----------
    self : RefSegmentP1
        a RefSegmentP1 object
    vertice : ndarray
        vertice of the cell

    Returns
    -------
    normal: ndarray
        Normal coordinate
    """

    t = vertice[0, :] - vertice[1, :]
    n = np.cross([t[0], t[1], 0], [0, 0, 1])[0:2]
    n = n / np.linalg.norm(n)

    return n



# -*- coding: utf-8 -*-
from numpy import pi, angle, exp

from ....Classes.Circle import Circle
from ....Classes.SurfLine import SurfLine
from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment


def build_geometry(self, sym=1, alpha=0, delta=0):
    """Build the geometry of the LamSlotMulti object

    Parameters
    ----------
    self : LamSlotMulti
        a LamSlotMulti object
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the lamination

    """

    # getting Number of Slot
    Zs = self.get_Zs()

    # Check for symmetry
    assert (Zs % sym) == 0

    if self.is_stator:
        ll = "Stator"  # Label lamination
    else:
        ll = "Rotor"
    if self.is_internal:
        ls = "Ext"  # label for the slot
        ly = "Int"  # label for the yoke
    else:
        ls = "Int"
        ly = "Ext"

    Ryoke = self.get_Ryoke()
    Rbo = self.get_Rbo()
    H_yoke = self.comp_height_yoke()

    bore_desc = self.get_bore_desc(sym=sym)
    bore_list = list()
    for bore in bore_desc:
        if type(bore["obj"]) is Arc1:
            bore_list.append(bore["obj"])
        else:
            lines = bore["obj"].build_geometry()
            for line in lines:
                line.rotate((bore["begin_angle"] + bore["end_angle"]) / 2)
            bore_list.extend(lines)

    # Create the lamination surface(s)
    surf_list = list()
    if sym == 1:  # Complete lamination
        # Create Slot surface
        surf_slot = SurfLine(
            line_list=bore_list, label="Lamination_" + ll + "_Bore_" + ls
        )
        if self.is_internal:
            surf_slot.point_ref = Ryoke + (H_yoke / 2)
        else:
            surf_slot.point_ref = Ryoke - (H_yoke / 2)
        # Create yoke circle surface
        if Ryoke > 0:
            surf_yoke = Circle(
                radius=Ryoke,
                label="Lamination_" + ll + "_Yoke_" + ly,
                line_label=ll + "_Yoke_Radius",
                center=0,
            )
        # The order matters when plotting
        if self.is_internal:
            surf_list.append(surf_slot)
            if Ryoke > 0:
                surf_list.append(surf_yoke)
        else:
            surf_yoke.point_ref = None  # No need to set the surface
            surf_list.append(surf_yoke)
            surf_list.append(surf_slot)
    else:  # Only one surface
        # Add the Yoke part
        Zy1 = Ryoke
        Zy2 = Ryoke * exp(1j * 2 * pi / sym)
        bore_list.append(
            Segment(Rbo * exp(1j * (2 * pi / sym)), Zy2, label=ll + "_Yoke_Side_Right")
        )
        if Ryoke > 0:
            # For internal lamination Ryoke can be 0
            bore_list.append(
                Arc1(
                    begin=Zy2,
                    end=Zy1,
                    radius=-Ryoke,
                    is_trigo_direction=False,
                    label=ll + "_Yoke_Radius",
                )
            )
        bore_list.append(Segment(Zy1, Rbo, label=ll + "_Yoke_Side_Left"))
        # Create a Surface for the slot
        if self.is_internal:
            point_ref = (Ryoke + H_yoke / 2) * exp(1j * pi / sym)
        else:
            point_ref = (Ryoke - H_yoke / 2) * exp(1j * pi / sym)
        surf_slot = SurfLine(
            line_list=bore_list,
            label="Lamination_" + ll + "_Bore_" + ls,
            point_ref=point_ref,
        )
        surf_list.append(surf_slot)

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    # Adding the ventilation surfaces
    for vent in self.axial_vent:
        surf_list += vent.build_geometry(
            sym=sym, alpha=alpha, delta=delta, is_stator=self.is_stator
        )
    return surf_list



from os import remove
from os.path import join

from ....Classes.Material import Material
from ....Functions.load import load_matlib
from ....Functions.Material.compare_material import compare_material
from ....Functions.Material.replace_material_pyleecan_obj import (
    replace_material_pyleecan_obj,
)
from ....definitions import config_dict
from ....loggers import GUI_LOG_NAME
from logging import getLogger
from PySide2.QtCore import QObject, Signal


class MatLib(QObject):
    saveNeeded = Signal()

    def __init__(self, path=None):
        """MatLib contains the material library and the specific machine materials."""
        # Dict containing the material library and the specific machine materials
        self.dict_mat = {
            "RefMatLib": [],  # Reference Material Library
            "MachineMatLib": [],  # Machine-specific materials
        }

        # Reference material library path
        self.ref_path = path

        # Load the material reference from the path
        if path:
            self.load_mat_ref(path)

        self.machine = None

    def load_mat_ref(self, path):
        """Remove the reference materials and load materials from the path

        Parameters
        ----------
        self: MatLib
        path: str
            new material library path
        """
        # Load a complete new matlib
        self.dict_mat["RefMatLib"] = load_matlib(path)

        # Remove MATLIB_DIR at the beginning of the material path
        for mat in self.dict_mat["RefMatLib"]:
            if config_dict["MAIN"]["MATLIB_DIR"].replace(
                "\\", " / "
            ) in mat.path.replace("\\", "/"):
                mat.path = mat.path.replace("\\", "/")[
                    len(config_dict["MAIN"]["MATLIB_DIR"].replace("\\", " / ")) + 1 :
                ]
            mat.path = mat.path.replace("\\", "/").strip("/")

        self.ref_path = path

    def add_machine_mat(self, machine):
        """
        Add machine materials if they are not in the MatLib

        Parameters
        ----------
        self: MatLib

        machine: Machine
            Machine containing material to add

        Returns
        -------
        is_change: bool
            Machine has changed
        """
        # Set the machine
        self.machine = machine

        # No change by default
        is_change = False

        # Remove previous machine materials
        self.dict_mat["MachineMatLib"] = []

        list_mach_mat = machine.get_material_list()
        if not isinstance(list_mach_mat, list):
            list_mach_mat = [list_mach_mat]

        # Copy the matlib and remove the name and the path to compare the materials
        list_mat_noname = [
            Material(init_dict=mat.as_dict()) for mat in self.dict_mat["RefMatLib"]
        ]
        for mat in list_mat_noname:
            mat.name = ""
            mat.path = ""

        # Add the material to machine material if it is not already contained in the Matlib
        for material in list_mach_mat:
            # Avoid materials from the default machine
            if material.is_isotropic == None or material == Material():
                continue

            # Store name and path to compare other attributes
            name = material.name
            path = material.path
            material.name = ""
            material.path = ""

            # Add the material if not in matlib
            if material not in list_mat_noname:
                material.name = name
                material.path = path
                self.dict_mat["MachineMatLib"].append(material)
                self.check_material_duplicated_name("MachineMatLib", -1)

                is_change_mat = replace_material_pyleecan_obj(
                    machine, material, material, False
                )
                is_change = is_change or is_change_mat

            # Replace the material in the machine by the MatLib one
            else:
                material.name = name
                material.path = path
                for mat in self.dict_mat["RefMatLib"]:
                    # Find the material in the matlib
                    if material == mat:  # Same name and path
                        # No need to change the material
                        break
                    elif compare_material(material, mat):
                        # Replace material by mat in machine
                        is_change_mat = replace_material_pyleecan_obj(
                            machine, material, mat, False
                        )

                        is_change = is_change or is_change_mat
                        break

        return is_change

    def move_mach_mat_to_ref(self, key, index):
        """
        Move a machine material into the matlib reference

        Parameters
        ----------
        self: MatLib
        key: str
            machine name to move the right material
        index: int
            position of the machine material in its machine materials list
        """

        # Move the material into the Material library
        mat = self.dict_mat["MachineMatLib"].pop(index)
        self.dict_mat["RefMatLib"].append(mat)

        # Set the path of the new matlib material
        new_path = self.dict_mat["RefMatLib"][-1].name + ".json"
        self.dict_mat["RefMatLib"][-1].path = new_path

        self.dict_mat["RefMatLib"][-1].save(
            join(config_dict["MAIN"]["MATLIB_DIR"], new_path)
        )

    def move_ref_mat_to_mach(self, key, index):
        """
        Move a material from the reference material library to
        the machine materials

        self: MatLib
        key: str
            key to select the right machine list
        index: int
            material to move position in the reference material list
        """

        mat_to_move = self.dict_mat["RefMatLib"].pop(index)

        # Get material logger
        logger = mat_to_move.get_logger()

        # Try to remove the material from the material library
        try:
            remove(join(config_dict["MAIN"]["MATLIB_DIR"], mat_to_move.path))
            logger.info(
                'Delete material "{}" file: {}'.format(
                    mat_to_move.name,
                    join(config_dict["MAIN"]["MATLIB_DIR"], mat_to_move.path),
                )
            )
        except FileNotFoundError:
            logger.warning(
                'Couldn\'t delete material "{}" file: {}'.format(
                    mat_to_move.name,
                    join(config_dict["MAIN"]["MATLIB_DIR"], mat_to_move.path),
                )
            )

        self.dict_mat[key].append(mat_to_move)

    def delete_material(self, key, index):
        """
        Remove the material and delete the material file

        Parameters
        ----------
        self: MatLib

        key: str
            List selector in the dict : RefMatLib or MachineMatLib TODO MachineName in the future

        index: int
            position of the material to remove
        """
        # Get the material
        mat_to_del = self.dict_mat[key].pop(index)

        # Delete the material file if the material is in the Reference Material Library
        if key == "RefMatLib":
            getLogger(GUI_LOG_NAME).info("Deleting the material: " + mat_to_del.path)
            remove(
                join(config_dict["MAIN"]["MATLIB_DIR"], mat_to_del.path)
            )  # Delete the file

    def check_material_duplicated_name(self, key, index):
        """
        Check if a material name is already used, modify it and its path if needed

        Parameters
        ----------
        self: MatLib

        key: str
            List selector in the dict : RefMatLib or MachineMatLib TODO MachineName in the future
        index: int
            position of the material to check
        """
        if index == -1:
            index += len(self.dict_mat[key])

        is_renamed = False
        mat_duplicated = True

        # Get the material name
        name = self.dict_mat[key][index].name
        if name == None:
            is_renamed = True
            name = "Untitled"

        list_mat_name = []
        for key_iter in self.dict_mat.keys():
            for i, mat in enumerate(self.dict_mat[key_iter]):
                if i != index or key_iter != key:
                    list_mat_name.append(mat.name)

        # Browse the material list name and change the name while the name exist
        while mat_duplicated:
            mat_duplicated = False
            for ref_mat_name in list_mat_name:
                if ref_mat_name == name:
                    is_renamed = True
                    if name.endswith(")") and "(" in name:
                        idx = name.rfind("(")
                        num = int(name[idx + 1 : -1]) + 1
                        name = "{}({})".format(name[:idx], num)
                    else:
                        name += "(1)"
                    mat_duplicated = True
                    break

        # Change path to save the material
        if is_renamed:
            self.dict_mat[key][index].name = name
            path = self.dict_mat[key][index].path
            if path == None:
                path = ""
            else:
                path = path.replace("\\", " / ")

            # Remove the previous file if needed
            if key == "RefMatLib":
                try:
                    remove(path)
                except FileNotFoundError:
                    logger = self.dict_mat[key][index].get_logger()
                    logger.warning(
                        'Couldn\'t delete old material "{}" file: {}'.format(name, path)
                    )

            idx = path.rfind("/")
            if idx == -1:
                self.dict_mat[key][index].path = (
                    self.dict_mat[key][index].name + ".json"
                )
            else:
                self.dict_mat[key][index].path = (
                    path[: idx + 1] + self.dict_mat[key][index].name + ".json"
                )

    def add_new_mat_ref(self, material):
        """
        Add a new material in the reference matlib

        Parameters
        ----------
        self: MatLib
        material: Material
            material to add
        """
        self.dict_mat["RefMatLib"].append(material)

        # Check if the material is duplicated
        self.check_material_duplicated_name("RefMatLib", -1)

        material.path = material.name + ".json"

        getLogger(GUI_LOG_NAME).info("Creating the material: " + material.path)

        # Saving the material
        material.save(join(config_dict["MAIN"]["MATLIB_DIR"], material.path))

    def add_new_mat_mach(self, material):
        """
        Add a new material in the machine materials

        Parameters
        ----------
        self: MatLib
        material: Material
            material to add
        """
        self.dict_mat["MachineMatLib"].append(material)
        self.check_material_duplicated_name("MachineMatLib", -1)

    def replace_material(self, key, index, material, save=True):
        """
        Replace a material

        Parameters
        ----------
        self: MatLib
        key: str
            dict_mat key to select the right material list
        index: int
            index of the material to replace
        material: Material
            new material
        save: bool
            save modification in the MatLib file

        Returns
        -------

        is_change: bool
            Machine has been change
        """
        is_change = False
        # Replace the material in the current machine
        if self.machine != None:
            is_change = replace_material_pyleecan_obj(
                self.machine, self.dict_mat[key][index], material, comp_name_path=False
            )

        # Material Library
        if key == "RefMatLib":
            if save:
                try:
                    # Delete the previous material
                    remove(
                        join(
                            config_dict["MAIN"]["MATLIB_DIR"],
                            self.dict_mat["RefMatLib"][index].path,
                        )
                    )
                except FileNotFoundError:
                    logger = material.get_logger()
                    logger.warning(
                        'Couldn\'t remove machine "{}" file: {}'.format(
                            material.name,
                            join(
                                config_dict["MAIN"]["MATLIB_DIR"],
                                self.dict_mat["RefMatLib"][index].path,
                            ),
                        )
                    )

            # Replace the material
            self.dict_mat[key][index] = material

            # Check its name
            self.check_material_duplicated_name(key, index)

            # Set its path
            material.path = material.name + ".json"

            if save:
                # Save it
                material.save(join(config_dict["MAIN"]["MATLIB_DIR"], material.path))

        # Machine material
        else:
            # Replace the material
            self.dict_mat[key][index] = material

            # Check its name
            self.check_material_duplicated_name(key, index)

        return is_change



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Material/MatHT.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Material/MatHT
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError


class MatHT(FrozenClass):
    """Material Heat Transfer properties"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        lambda_x=1,
        lambda_y=1,
        lambda_z=1,
        Cp=1,
        alpha=0.00393,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "lambda_x" in list(init_dict.keys()):
                lambda_x = init_dict["lambda_x"]
            if "lambda_y" in list(init_dict.keys()):
                lambda_y = init_dict["lambda_y"]
            if "lambda_z" in list(init_dict.keys()):
                lambda_z = init_dict["lambda_z"]
            if "Cp" in list(init_dict.keys()):
                Cp = init_dict["Cp"]
            if "alpha" in list(init_dict.keys()):
                alpha = init_dict["alpha"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.lambda_x = lambda_x
        self.lambda_y = lambda_y
        self.lambda_z = lambda_z
        self.Cp = Cp
        self.alpha = alpha

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        MatHT_str = ""
        if self.parent is None:
            MatHT_str += "parent = None " + linesep
        else:
            MatHT_str += "parent = " + str(type(self.parent)) + " object" + linesep
        MatHT_str += "lambda_x = " + str(self.lambda_x) + linesep
        MatHT_str += "lambda_y = " + str(self.lambda_y) + linesep
        MatHT_str += "lambda_z = " + str(self.lambda_z) + linesep
        MatHT_str += "Cp = " + str(self.Cp) + linesep
        MatHT_str += "alpha = " + str(self.alpha) + linesep
        return MatHT_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.lambda_x != self.lambda_x:
            return False
        if other.lambda_y != self.lambda_y:
            return False
        if other.lambda_z != self.lambda_z:
            return False
        if other.Cp != self.Cp:
            return False
        if other.alpha != self.alpha:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._lambda_x != self._lambda_x:
            diff_list.append(name + ".lambda_x")
        if other._lambda_y != self._lambda_y:
            diff_list.append(name + ".lambda_y")
        if other._lambda_z != self._lambda_z:
            diff_list.append(name + ".lambda_z")
        if other._Cp != self._Cp:
            diff_list.append(name + ".Cp")
        if other._alpha != self._alpha:
            diff_list.append(name + ".alpha")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.lambda_x)
        S += getsizeof(self.lambda_y)
        S += getsizeof(self.lambda_z)
        S += getsizeof(self.Cp)
        S += getsizeof(self.alpha)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        MatHT_dict = dict()
        MatHT_dict["lambda_x"] = self.lambda_x
        MatHT_dict["lambda_y"] = self.lambda_y
        MatHT_dict["lambda_z"] = self.lambda_z
        MatHT_dict["Cp"] = self.Cp
        MatHT_dict["alpha"] = self.alpha
        # The class name is added to the dict for deserialisation purpose
        MatHT_dict["__class__"] = "MatHT"
        return MatHT_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.lambda_x = None
        self.lambda_y = None
        self.lambda_z = None
        self.Cp = None
        self.alpha = None

    def _get_lambda_x(self):
        """getter of lambda_x"""
        return self._lambda_x

    def _set_lambda_x(self, value):
        """setter of lambda_x"""
        check_var("lambda_x", value, "float", Vmin=0)
        self._lambda_x = value

    lambda_x = property(
        fget=_get_lambda_x,
        fset=_set_lambda_x,
        doc=u"""thermal conductivity (XY is lamination plane, Z is rotation axis)

        :Type: float
        :min: 0
        """,
    )

    def _get_lambda_y(self):
        """getter of lambda_y"""
        return self._lambda_y

    def _set_lambda_y(self, value):
        """setter of lambda_y"""
        check_var("lambda_y", value, "float", Vmin=0)
        self._lambda_y = value

    lambda_y = property(
        fget=_get_lambda_y,
        fset=_set_lambda_y,
        doc=u"""thermal conductivity (XY is lamination plane, Z is rotation axis)

        :Type: float
        :min: 0
        """,
    )

    def _get_lambda_z(self):
        """getter of lambda_z"""
        return self._lambda_z

    def _set_lambda_z(self, value):
        """setter of lambda_z"""
        check_var("lambda_z", value, "float", Vmin=0)
        self._lambda_z = value

    lambda_z = property(
        fget=_get_lambda_z,
        fset=_set_lambda_z,
        doc=u"""thermal conductivity (XY is lamination plane, Z is rotation axis)

        :Type: float
        :min: 0
        """,
    )

    def _get_Cp(self):
        """getter of Cp"""
        return self._Cp

    def _set_Cp(self, value):
        """setter of Cp"""
        check_var("Cp", value, "float", Vmin=0)
        self._Cp = value

    Cp = property(
        fget=_get_Cp,
        fset=_set_Cp,
        doc=u"""specific heat capacity

        :Type: float
        :min: 0
        """,
    )

    def _get_alpha(self):
        """getter of alpha"""
        return self._alpha

    def _set_alpha(self, value):
        """setter of alpha"""
        check_var("alpha", value, "float", Vmin=0)
        self._alpha = value

    alpha = property(
        fget=_get_alpha,
        fset=_set_alpha,
        doc=u"""thermal expansion coefficient

        :Type: float
        :min: 0
        """,
    )



# -*- coding: utf-8 -*-

# File generated according to SLamParam.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from .....GUI.Tools.FloatEdit import FloatEdit
from .....GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SLamParam(object):
    def setupUi(self, SLamParam):
        if not SLamParam.objectName():
            SLamParam.setObjectName(u"SLamParam")
        SLamParam.resize(650, 550)
        SLamParam.setMinimumSize(QSize(650, 550))
        self.main_layout = QVBoxLayout(SLamParam)
        self.main_layout.setObjectName(u"main_layout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.in_L1 = QLabel(SLamParam)
        self.in_L1.setObjectName(u"in_L1")

        self.horizontalLayout.addWidget(self.in_L1)

        self.lf_L1 = FloatEdit(SLamParam)
        self.lf_L1.setObjectName(u"lf_L1")
        self.lf_L1.setMaximumSize(QSize(100, 16777215))

        self.horizontalLayout.addWidget(self.lf_L1)

        self.unit_L1 = QLabel(SLamParam)
        self.unit_L1.setObjectName(u"unit_L1")

        self.horizontalLayout.addWidget(self.unit_L1)

        self.horizontalSpacer_8 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout.addItem(self.horizontalSpacer_8)

        self.main_layout.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.in_Kf1 = QLabel(SLamParam)
        self.in_Kf1.setObjectName(u"in_Kf1")

        self.horizontalLayout_2.addWidget(self.in_Kf1)

        self.lf_Kf1 = FloatEdit(SLamParam)
        self.lf_Kf1.setObjectName(u"lf_Kf1")
        self.lf_Kf1.setMaximumSize(QSize(100, 25))

        self.horizontalLayout_2.addWidget(self.lf_Kf1)

        self.w_mat = WMatSelect(SLamParam)
        self.w_mat.setObjectName(u"w_mat")
        self.w_mat.setMinimumSize(QSize(100, 0))

        self.horizontalLayout_2.addWidget(self.w_mat)

        self.horizontalSpacer_2 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer_2)

        self.main_layout.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.g_ax_vent = QGroupBox(SLamParam)
        self.g_ax_vent.setObjectName(u"g_ax_vent")
        self.g_ax_vent.setMinimumSize(QSize(300, 300))
        self.g_ax_vent.setCheckable(True)
        self.g_ax_vent.setChecked(False)
        self.verticalLayout_3 = QVBoxLayout(self.g_ax_vent)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.label = QLabel(self.g_ax_vent)
        self.label.setObjectName(u"label")
        self.label.setPixmap(
            QPixmap(u":/images/images/MachineSetup/LamParam/CircVentDuct.png")
        )
        self.label.setScaledContents(True)

        self.verticalLayout_3.addWidget(self.label)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.out_axial_duct = QLabel(self.g_ax_vent)
        self.out_axial_duct.setObjectName(u"out_axial_duct")

        self.horizontalLayout_5.addWidget(self.out_axial_duct)

        self.b_axial_duct = QPushButton(self.g_ax_vent)
        self.b_axial_duct.setObjectName(u"b_axial_duct")

        self.horizontalLayout_5.addWidget(self.b_axial_duct)

        self.horizontalSpacer_4 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_5.addItem(self.horizontalSpacer_4)

        self.verticalLayout_3.addLayout(self.horizontalLayout_5)

        self.horizontalLayout_3.addWidget(self.g_ax_vent)

        self.g_rad_vent = QGroupBox(SLamParam)
        self.g_rad_vent.setObjectName(u"g_rad_vent")
        self.g_rad_vent.setMinimumSize(QSize(0, 300))
        self.g_rad_vent.setFlat(False)
        self.g_rad_vent.setCheckable(True)
        self.g_rad_vent.setChecked(False)
        self.verticalLayout_2 = QVBoxLayout(self.g_rad_vent)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.img_rad_duct = QLabel(self.g_rad_vent)
        self.img_rad_duct.setObjectName(u"img_rad_duct")
        self.img_rad_duct.setMinimumSize(QSize(300, 0))
        self.img_rad_duct.setPixmap(
            QPixmap(u":/images/images/MachineSetup/LamParam/RadVentDuct.png")
        )
        self.img_rad_duct.setScaledContents(True)

        self.verticalLayout_2.addWidget(self.img_rad_duct)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.in_Nrvd = QLabel(self.g_rad_vent)
        self.in_Nrvd.setObjectName(u"in_Nrvd")

        self.horizontalLayout_4.addWidget(self.in_Nrvd)

        self.si_Nrvd = QSpinBox(self.g_rad_vent)
        self.si_Nrvd.setObjectName(u"si_Nrvd")
        self.si_Nrvd.setMaximum(999)
        self.si_Nrvd.setValue(0)

        self.horizontalLayout_4.addWidget(self.si_Nrvd)

        self.in_Wrvd = QLabel(self.g_rad_vent)
        self.in_Wrvd.setObjectName(u"in_Wrvd")

        self.horizontalLayout_4.addWidget(self.in_Wrvd)

        self.lf_Wrvd = FloatEdit(self.g_rad_vent)
        self.lf_Wrvd.setObjectName(u"lf_Wrvd")
        self.lf_Wrvd.setMaximumSize(QSize(100, 16777215))

        self.horizontalLayout_4.addWidget(self.lf_Wrvd)

        self.unit_Wrvd = QLabel(self.g_rad_vent)
        self.unit_Wrvd.setObjectName(u"unit_Wrvd")

        self.horizontalLayout_4.addWidget(self.unit_Wrvd)

        self.horizontalSpacer_6 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_4.addItem(self.horizontalSpacer_6)

        self.verticalLayout_2.addLayout(self.horizontalLayout_4)

        self.out_length = QLabel(self.g_rad_vent)
        self.out_length.setObjectName(u"out_length")
        self.out_length.setMinimumSize(QSize(250, 0))
        self.out_length.setMaximumSize(QSize(300, 16777215))

        self.verticalLayout_2.addWidget(self.out_length)

        self.horizontalLayout_3.addWidget(self.g_rad_vent)

        self.main_layout.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.horizontalSpacer_7 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_6.addItem(self.horizontalSpacer_7)

        self.b_plot = QPushButton(SLamParam)
        self.b_plot.setObjectName(u"b_plot")

        self.horizontalLayout_6.addWidget(self.b_plot)

        self.b_previous = QPushButton(SLamParam)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout_6.addWidget(self.b_previous)

        self.b_next = QPushButton(SLamParam)
        self.b_next.setObjectName(u"b_next")

        self.horizontalLayout_6.addWidget(self.b_next)

        self.main_layout.addLayout(self.horizontalLayout_6)

        QWidget.setTabOrder(self.lf_L1, self.lf_Kf1)
        QWidget.setTabOrder(self.lf_Kf1, self.b_next)
        QWidget.setTabOrder(self.b_next, self.si_Nrvd)
        QWidget.setTabOrder(self.si_Nrvd, self.lf_Wrvd)

        self.retranslateUi(SLamParam)

        QMetaObject.connectSlotsByName(SLamParam)

    # setupUi

    def retranslateUi(self, SLamParam):
        SLamParam.setWindowTitle(QCoreApplication.translate("SLamParam", u"Form", None))
        self.in_L1.setText(QCoreApplication.translate("SLamParam", u"L1 :", None))
        self.unit_L1.setText(QCoreApplication.translate("SLamParam", u"m", None))
        self.in_Kf1.setText(QCoreApplication.translate("SLamParam", u"Kf1 :", None))
        self.g_ax_vent.setTitle(
            QCoreApplication.translate("SLamParam", u"Axial Ventilation Ducts", None)
        )
        self.label.setText("")
        self.out_axial_duct.setText(
            QCoreApplication.translate("SLamParam", u"Axial : 0 set (0 ducts)", None)
        )
        self.b_axial_duct.setText(
            QCoreApplication.translate("SLamParam", u"set axial ducts", None)
        )
        self.g_rad_vent.setTitle(
            QCoreApplication.translate("SLamParam", u"Radial Ventilation Ducts", None)
        )
        self.img_rad_duct.setText("")
        self.in_Nrvd.setText(QCoreApplication.translate("SLamParam", u"Nrvd :", None))
        self.in_Wrvd.setText(QCoreApplication.translate("SLamParam", u"Wrvd :", None))
        self.unit_Wrvd.setText(QCoreApplication.translate("SLamParam", u"m", None))
        self.out_length.setText(
            QCoreApplication.translate(
                "SLamParam", u"stator total length = L1 + Nrvd * Wrvd = ?", None
            )
        )
        self.b_plot.setText(QCoreApplication.translate("SLamParam", u"Preview", None))
        self.b_previous.setText(
            QCoreApplication.translate("SLamParam", u"Previous", None)
        )
        self.b_next.setText(QCoreApplication.translate("SLamParam", u"Next", None))

    # retranslateUi



# -*- coding: utf-8 -*-

import numpy as np


def is_exist(self, connectivity):
    """Check the existence of a cell defined by a connectivity (vector of points indices).
    The order of points indices does not matter.

    Parameters
    ----------
    self : CellMat
        an CellMat object
    connectivity : ndarray
        an array of node tags

    Returns
    -------
        bool
            True if the element already exist
    """

    # Check the existence of the element
    e = np.array([], dtype=int)
    for nd_tag in connectivity:
        e = np.concatenate((e, self.get_point2cell(nd_tag)))

    unique, unique_counts = np.unique(e, return_counts=True)
    for ie in range(len(unique)):
        if unique_counts[ie] == self.nb_pt_per_cell and unique_counts[ie] == len(
            connectivity
        ):
            # If this condition is valid, the element already exist
            return True
    return False



# -*- coding: utf-8 -*-

import numpy as np


def get_real_point(self, vertice, ref_pt, nb_ref_pt=1):
    """Return the coordinates in the cell of a point in the reference cell.

    Parameters
     ----------
    self : RefCell
         an RefCell object
    vertice : ndarray
        vertices of the cell
    ref_pt : ndarray
        ref point(s)
    nb_ref_pt : int
        nb of ref points

     Returns
     -------
    real_points : ndarray
        points coordinate

    """

    real_points = np.zeros((nb_ref_pt, 2))

    for ii in range(nb_ref_pt):
        if nb_ref_pt == 1:
            s = ref_pt[0]
        else:
            s = ref_pt[ii, 0]

        length = 2
        s = (s + 1) / length
        real_points[ii, :] = vertice[0, :] + s * (vertice[1, :] - vertice[0, :])

    return real_points



from ....Methods.Geometry.Segment import PointTranslateSegmentError


def translate(self, Zt):
    """Translate the Segment object

    Parameters
    ----------
    self : Segment
        An Segment object

    Zt : complex
        Complex value for translation

    Returns
    -------
    None
    """
    if (
        not isinstance(Zt, complex)
        and not isinstance(Zt, int)
        and not isinstance(Zt, float)
    ):
        raise PointTranslateSegmentError(
            "The point must be a complex number or int or float"
        )

    # check if the Segment is correct
    self.check()

    # Modification from the translation of Segment
    self.begin = self.begin + Zt
    self.end = self.end + Zt



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/LamSlotWind.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/LamSlotWind
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .LamSlot import LamSlot

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.LamSlotWind.build_geometry import build_geometry
except ImportError as error:
    build_geometry = error

try:
    from ..Methods.Machine.LamSlotWind.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Machine.LamSlotWind.comp_masses import comp_masses
except ImportError as error:
    comp_masses = error

try:
    from ..Methods.Machine.LamSlotWind.comp_surfaces import comp_surfaces
except ImportError as error:
    comp_surfaces = error

try:
    from ..Methods.Machine.LamSlotWind.comp_volumes import comp_volumes
except ImportError as error:
    comp_volumes = error

try:
    from ..Methods.Machine.LamSlotWind.get_pole_pair_number import get_pole_pair_number
except ImportError as error:
    get_pole_pair_number = error

try:
    from ..Methods.Machine.LamSlotWind.get_name_phase import get_name_phase
except ImportError as error:
    get_name_phase = error

try:
    from ..Methods.Machine.LamSlotWind.plot import plot
except ImportError as error:
    plot = error

try:
    from ..Methods.Machine.LamSlotWind.plot_winding import plot_winding
except ImportError as error:
    plot_winding = error

try:
    from ..Methods.Machine.LamSlotWind.comp_fill_factor import comp_fill_factor
except ImportError as error:
    comp_fill_factor = error

try:
    from ..Methods.Machine.LamSlotWind.comp_output_geo import comp_output_geo
except ImportError as error:
    comp_output_geo = error

try:
    from ..Methods.Machine.LamSlotWind.get_polar_eq import get_polar_eq
except ImportError as error:
    get_polar_eq = error

try:
    from ..Methods.Machine.LamSlotWind.comp_wind_function import comp_wind_function
except ImportError as error:
    comp_wind_function = error

try:
    from ..Methods.Machine.LamSlotWind.plot_mmf_unit import plot_mmf_unit
except ImportError as error:
    plot_mmf_unit = error

try:
    from ..Methods.Machine.LamSlotWind.comp_resistance_wind import comp_resistance_wind
except ImportError as error:
    comp_resistance_wind = error

try:
    from ..Methods.Machine.LamSlotWind.comp_angle_d_axis import comp_angle_d_axis
except ImportError as error:
    comp_angle_d_axis = error

try:
    from ..Methods.Machine.LamSlotWind.comp_mmf_unit import comp_mmf_unit
except ImportError as error:
    comp_mmf_unit = error

try:
    from ..Methods.Machine.LamSlotWind.comp_rot_dir import comp_rot_dir
except ImportError as error:
    comp_rot_dir = error

try:
    from ..Methods.Machine.LamSlotWind.comp_lengths_winding import comp_lengths_winding
except ImportError as error:
    comp_lengths_winding = error

try:
    from ..Methods.Machine.LamSlotWind.comp_number_phase_eq import comp_number_phase_eq
except ImportError as error:
    comp_number_phase_eq = error

try:
    from ..Methods.Machine.LamSlotWind.comp_periodicity import comp_periodicity
except ImportError as error:
    comp_periodicity = error


from ._check import InitUnKnowClassError
from .Winding import Winding
from .Slot import Slot
from .Material import Material
from .Hole import Hole
from .Notch import Notch


class LamSlotWind(LamSlot):
    """Lamination with Slot filled with winding"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Machine.LamSlotWind.build_geometry
    if isinstance(build_geometry, ImportError):
        build_geometry = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method build_geometry: "
                    + str(build_geometry)
                )
            )
        )
    else:
        build_geometry = build_geometry
    # cf Methods.Machine.LamSlotWind.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use LamSlotWind method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Machine.LamSlotWind.comp_masses
    if isinstance(comp_masses, ImportError):
        comp_masses = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_masses: " + str(comp_masses)
                )
            )
        )
    else:
        comp_masses = comp_masses
    # cf Methods.Machine.LamSlotWind.comp_surfaces
    if isinstance(comp_surfaces, ImportError):
        comp_surfaces = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_surfaces: " + str(comp_surfaces)
                )
            )
        )
    else:
        comp_surfaces = comp_surfaces
    # cf Methods.Machine.LamSlotWind.comp_volumes
    if isinstance(comp_volumes, ImportError):
        comp_volumes = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_volumes: " + str(comp_volumes)
                )
            )
        )
    else:
        comp_volumes = comp_volumes
    # cf Methods.Machine.LamSlotWind.get_pole_pair_number
    if isinstance(get_pole_pair_number, ImportError):
        get_pole_pair_number = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method get_pole_pair_number: "
                    + str(get_pole_pair_number)
                )
            )
        )
    else:
        get_pole_pair_number = get_pole_pair_number
    # cf Methods.Machine.LamSlotWind.get_name_phase
    if isinstance(get_name_phase, ImportError):
        get_name_phase = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method get_name_phase: "
                    + str(get_name_phase)
                )
            )
        )
    else:
        get_name_phase = get_name_phase
    # cf Methods.Machine.LamSlotWind.plot
    if isinstance(plot, ImportError):
        plot = property(
            fget=lambda x: raise_(
                ImportError("Can't use LamSlotWind method plot: " + str(plot))
            )
        )
    else:
        plot = plot
    # cf Methods.Machine.LamSlotWind.plot_winding
    if isinstance(plot_winding, ImportError):
        plot_winding = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method plot_winding: " + str(plot_winding)
                )
            )
        )
    else:
        plot_winding = plot_winding
    # cf Methods.Machine.LamSlotWind.comp_fill_factor
    if isinstance(comp_fill_factor, ImportError):
        comp_fill_factor = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_fill_factor: "
                    + str(comp_fill_factor)
                )
            )
        )
    else:
        comp_fill_factor = comp_fill_factor
    # cf Methods.Machine.LamSlotWind.comp_output_geo
    if isinstance(comp_output_geo, ImportError):
        comp_output_geo = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_output_geo: "
                    + str(comp_output_geo)
                )
            )
        )
    else:
        comp_output_geo = comp_output_geo
    # cf Methods.Machine.LamSlotWind.get_polar_eq
    if isinstance(get_polar_eq, ImportError):
        get_polar_eq = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method get_polar_eq: " + str(get_polar_eq)
                )
            )
        )
    else:
        get_polar_eq = get_polar_eq
    # cf Methods.Machine.LamSlotWind.comp_wind_function
    if isinstance(comp_wind_function, ImportError):
        comp_wind_function = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_wind_function: "
                    + str(comp_wind_function)
                )
            )
        )
    else:
        comp_wind_function = comp_wind_function
    # cf Methods.Machine.LamSlotWind.plot_mmf_unit
    if isinstance(plot_mmf_unit, ImportError):
        plot_mmf_unit = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method plot_mmf_unit: " + str(plot_mmf_unit)
                )
            )
        )
    else:
        plot_mmf_unit = plot_mmf_unit
    # cf Methods.Machine.LamSlotWind.comp_resistance_wind
    if isinstance(comp_resistance_wind, ImportError):
        comp_resistance_wind = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_resistance_wind: "
                    + str(comp_resistance_wind)
                )
            )
        )
    else:
        comp_resistance_wind = comp_resistance_wind
    # cf Methods.Machine.LamSlotWind.comp_angle_d_axis
    if isinstance(comp_angle_d_axis, ImportError):
        comp_angle_d_axis = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_angle_d_axis: "
                    + str(comp_angle_d_axis)
                )
            )
        )
    else:
        comp_angle_d_axis = comp_angle_d_axis
    # cf Methods.Machine.LamSlotWind.comp_mmf_unit
    if isinstance(comp_mmf_unit, ImportError):
        comp_mmf_unit = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_mmf_unit: " + str(comp_mmf_unit)
                )
            )
        )
    else:
        comp_mmf_unit = comp_mmf_unit
    # cf Methods.Machine.LamSlotWind.comp_rot_dir
    if isinstance(comp_rot_dir, ImportError):
        comp_rot_dir = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_rot_dir: " + str(comp_rot_dir)
                )
            )
        )
    else:
        comp_rot_dir = comp_rot_dir
    # cf Methods.Machine.LamSlotWind.comp_lengths_winding
    if isinstance(comp_lengths_winding, ImportError):
        comp_lengths_winding = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_lengths_winding: "
                    + str(comp_lengths_winding)
                )
            )
        )
    else:
        comp_lengths_winding = comp_lengths_winding
    # cf Methods.Machine.LamSlotWind.comp_number_phase_eq
    if isinstance(comp_number_phase_eq, ImportError):
        comp_number_phase_eq = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_number_phase_eq: "
                    + str(comp_number_phase_eq)
                )
            )
        )
    else:
        comp_number_phase_eq = comp_number_phase_eq
    # cf Methods.Machine.LamSlotWind.comp_periodicity
    if isinstance(comp_periodicity, ImportError):
        comp_periodicity = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use LamSlotWind method comp_periodicity: "
                    + str(comp_periodicity)
                )
            )
        )
    else:
        comp_periodicity = comp_periodicity
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Ksfill=None,
        winding=-1,
        slot=-1,
        L1=0.35,
        mat_type=-1,
        Nrvd=0,
        Wrvd=0,
        Kf1=0.95,
        is_internal=True,
        Rint=0,
        Rext=1,
        is_stator=True,
        axial_vent=-1,
        notch=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Ksfill" in list(init_dict.keys()):
                Ksfill = init_dict["Ksfill"]
            if "winding" in list(init_dict.keys()):
                winding = init_dict["winding"]
            if "slot" in list(init_dict.keys()):
                slot = init_dict["slot"]
            if "L1" in list(init_dict.keys()):
                L1 = init_dict["L1"]
            if "mat_type" in list(init_dict.keys()):
                mat_type = init_dict["mat_type"]
            if "Nrvd" in list(init_dict.keys()):
                Nrvd = init_dict["Nrvd"]
            if "Wrvd" in list(init_dict.keys()):
                Wrvd = init_dict["Wrvd"]
            if "Kf1" in list(init_dict.keys()):
                Kf1 = init_dict["Kf1"]
            if "is_internal" in list(init_dict.keys()):
                is_internal = init_dict["is_internal"]
            if "Rint" in list(init_dict.keys()):
                Rint = init_dict["Rint"]
            if "Rext" in list(init_dict.keys()):
                Rext = init_dict["Rext"]
            if "is_stator" in list(init_dict.keys()):
                is_stator = init_dict["is_stator"]
            if "axial_vent" in list(init_dict.keys()):
                axial_vent = init_dict["axial_vent"]
            if "notch" in list(init_dict.keys()):
                notch = init_dict["notch"]
        # Set the properties (value check and convertion are done in setter)
        self.Ksfill = Ksfill
        self.winding = winding
        # Call LamSlot init
        super(LamSlotWind, self).__init__(
            slot=slot,
            L1=L1,
            mat_type=mat_type,
            Nrvd=Nrvd,
            Wrvd=Wrvd,
            Kf1=Kf1,
            is_internal=is_internal,
            Rint=Rint,
            Rext=Rext,
            is_stator=is_stator,
            axial_vent=axial_vent,
            notch=notch,
        )
        # The class is frozen (in LamSlot init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        LamSlotWind_str = ""
        # Get the properties inherited from LamSlot
        LamSlotWind_str += super(LamSlotWind, self).__str__()
        LamSlotWind_str += "Ksfill = " + str(self.Ksfill) + linesep
        if self.winding is not None:
            tmp = self.winding.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            LamSlotWind_str += "winding = " + tmp
        else:
            LamSlotWind_str += "winding = None" + linesep + linesep
        return LamSlotWind_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from LamSlot
        if not super(LamSlotWind, self).__eq__(other):
            return False
        if other.Ksfill != self.Ksfill:
            return False
        if other.winding != self.winding:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from LamSlot
        diff_list.extend(super(LamSlotWind, self).compare(other, name=name))
        if other._Ksfill != self._Ksfill:
            diff_list.append(name + ".Ksfill")
        if (other.winding is None and self.winding is not None) or (
            other.winding is not None and self.winding is None
        ):
            diff_list.append(name + ".winding None mismatch")
        elif self.winding is not None:
            diff_list.extend(
                self.winding.compare(other.winding, name=name + ".winding")
            )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from LamSlot
        S += super(LamSlotWind, self).__sizeof__()
        S += getsizeof(self.Ksfill)
        S += getsizeof(self.winding)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from LamSlot
        LamSlotWind_dict = super(LamSlotWind, self).as_dict(**kwargs)
        LamSlotWind_dict["Ksfill"] = self.Ksfill
        if self.winding is None:
            LamSlotWind_dict["winding"] = None
        else:
            LamSlotWind_dict["winding"] = self.winding.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        LamSlotWind_dict["__class__"] = "LamSlotWind"
        return LamSlotWind_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Ksfill = None
        if self.winding is not None:
            self.winding._set_None()
        # Set to None the properties inherited from LamSlot
        super(LamSlotWind, self)._set_None()

    def _get_Ksfill(self):
        """getter of Ksfill"""
        return self._Ksfill

    def _set_Ksfill(self, value):
        """setter of Ksfill"""
        check_var("Ksfill", value, "float", Vmin=0, Vmax=1)
        self._Ksfill = value

    Ksfill = property(
        fget=_get_Ksfill,
        fset=_set_Ksfill,
        doc=u"""Imposed Slot Fill factor (if None, will be computed according to the winding and the slot)

        :Type: float
        :min: 0
        :max: 1
        """,
    )

    def _get_winding(self):
        """getter of winding"""
        return self._winding

    def _set_winding(self, value):
        """setter of winding"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "winding"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Winding()
        check_var("winding", value, "Winding")
        self._winding = value

        if self._winding is not None:
            self._winding.parent = self

    winding = property(
        fget=_get_winding,
        fset=_set_winding,
        doc=u"""Lamination's Winding

        :Type: Winding
        """,
    )



# -*- coding: utf-8 -*-


def check(self):
    """Check that the Slot object is correct

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    None
    """
    pass



def comp_lengths_winding(self):
    """Compute the lengths of the Lamination's Winding.
    - Lwtot : total length of lamination winding incl. end-windings and
    radial ventilation ducts [m].
    - Lwact : active length of lamination winding excl. end-windings and
    radial ventilation ducts [m].
    - Lewt : total end-winding length [m].
    - Lew : end-winding length on one side for a half-turn
    - Lwvent : length of lamination winding in the radial ventilation ducts [m]

    Parameters
    ----------
    self: LamSlotWind
        a LamSlotWind object
    Returns
    -------
    L_dict: dict
        Dictionnary of the length (Lwtot, Lwact, Lew, Lwvent)
    """

    # length of the stack including ventilation ducts
    L1vd = self.comp_length()

    # end-winding length on one side for a half-turn
    Lew = self.winding.comp_length_endwinding()

    # total end-winding length
    Ntspc = self.winding.comp_Ntspc(self.slot.Zs)
    qb = self.comp_number_phase_eq()
    Lewt = qb * Ntspc * self.winding.Npcpp * 4 * Lew

    # average length of a lamination winding half-turn (one "go" conductor
    # without "return" conductor)
    Lwht = L1vd + 2 * Lew

    # total length of lamination winding incl. end windings [m]
    Lwtot = qb * Ntspc * self.winding.Npcpp * 2 * Lwht

    # Active length of lamination winding excl. end windings and radial
    # ventilation duct [m]
    Lwact = qb * Ntspc * self.winding.Npcpp * 2 * self.L1

    # length of lamination winding in the radial ventilation duct [m]
    if self.Nrvd is None or self.Wrvd is None:
        Lwvent = 0
    else:
        Lwvent = qb * Ntspc * self.winding.Npcpp * 2 * self.Nrvd * self.Wrvd

    return {"Lwtot": Lwtot, "Lwact": Lwact, "Lewt": Lewt, "Lwvent": Lwvent, "Lew": Lew}



# -*- coding: utf-8 -*-


def comp_masses(self):
    """Compute the masses of the Lamination

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    M_dict: dict
        Lamination mass dictionnary (Mtot, Mlam, Mteeth, Myoke) [kg]

    """
    rho = self.mat_type.struct.rho
    V_dict = self.comp_volumes()

    M_dict = dict()
    M_dict["Myoke"] = V_dict["Vyoke"] * self.Kf1 * rho
    M_dict["Mteeth"] = V_dict["Vteeth"] * self.Kf1 * rho
    M_dict["Mlam"] = V_dict["Vlam"] * self.Kf1 * rho
    M_dict["Mtot"] = M_dict["Mlam"]

    return M_dict



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/VarLoad.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/VarLoad
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .VarSimu import VarSimu

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.VarLoad.get_elec_datakeeper import get_elec_datakeeper
except ImportError as error:
    get_elec_datakeeper = error

try:
    from ..Methods.Simulation.VarLoad.get_mag_datakeeper import get_mag_datakeeper
except ImportError as error:
    get_mag_datakeeper = error

try:
    from ..Methods.Simulation.VarLoad.get_force_datakeeper import get_force_datakeeper
except ImportError as error:
    get_force_datakeeper = error


from ._check import InitUnKnowClassError
from .DataKeeper import DataKeeper
from .VarSimu import VarSimu
from .Post import Post


class VarLoad(VarSimu):
    """Abstract class to generate multi-simulation by changing the operating point"""

    VERSION = 1
    NAME = "Variable Load"

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.VarLoad.get_elec_datakeeper
    if isinstance(get_elec_datakeeper, ImportError):
        get_elec_datakeeper = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarLoad method get_elec_datakeeper: "
                    + str(get_elec_datakeeper)
                )
            )
        )
    else:
        get_elec_datakeeper = get_elec_datakeeper
    # cf Methods.Simulation.VarLoad.get_mag_datakeeper
    if isinstance(get_mag_datakeeper, ImportError):
        get_mag_datakeeper = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarLoad method get_mag_datakeeper: "
                    + str(get_mag_datakeeper)
                )
            )
        )
    else:
        get_mag_datakeeper = get_mag_datakeeper
    # cf Methods.Simulation.VarLoad.get_force_datakeeper
    if isinstance(get_force_datakeeper, ImportError):
        get_force_datakeeper = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use VarLoad method get_force_datakeeper: "
                    + str(get_force_datakeeper)
                )
            )
        )
    else:
        get_force_datakeeper = get_force_datakeeper
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        name="",
        desc="",
        datakeeper_list=-1,
        is_keep_all_output=False,
        stop_if_error=False,
        var_simu=None,
        nb_simu=0,
        is_reuse_femm_file=True,
        postproc_list=-1,
        pre_keeper_postproc_list=None,
        post_keeper_postproc_list=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "datakeeper_list" in list(init_dict.keys()):
                datakeeper_list = init_dict["datakeeper_list"]
            if "is_keep_all_output" in list(init_dict.keys()):
                is_keep_all_output = init_dict["is_keep_all_output"]
            if "stop_if_error" in list(init_dict.keys()):
                stop_if_error = init_dict["stop_if_error"]
            if "var_simu" in list(init_dict.keys()):
                var_simu = init_dict["var_simu"]
            if "nb_simu" in list(init_dict.keys()):
                nb_simu = init_dict["nb_simu"]
            if "is_reuse_femm_file" in list(init_dict.keys()):
                is_reuse_femm_file = init_dict["is_reuse_femm_file"]
            if "postproc_list" in list(init_dict.keys()):
                postproc_list = init_dict["postproc_list"]
            if "pre_keeper_postproc_list" in list(init_dict.keys()):
                pre_keeper_postproc_list = init_dict["pre_keeper_postproc_list"]
            if "post_keeper_postproc_list" in list(init_dict.keys()):
                post_keeper_postproc_list = init_dict["post_keeper_postproc_list"]
        # Set the properties (value check and convertion are done in setter)
        # Call VarSimu init
        super(VarLoad, self).__init__(
            name=name,
            desc=desc,
            datakeeper_list=datakeeper_list,
            is_keep_all_output=is_keep_all_output,
            stop_if_error=stop_if_error,
            var_simu=var_simu,
            nb_simu=nb_simu,
            is_reuse_femm_file=is_reuse_femm_file,
            postproc_list=postproc_list,
            pre_keeper_postproc_list=pre_keeper_postproc_list,
            post_keeper_postproc_list=post_keeper_postproc_list,
        )
        # The class is frozen (in VarSimu init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        VarLoad_str = ""
        # Get the properties inherited from VarSimu
        VarLoad_str += super(VarLoad, self).__str__()
        return VarLoad_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from VarSimu
        if not super(VarLoad, self).__eq__(other):
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from VarSimu
        diff_list.extend(super(VarLoad, self).compare(other, name=name))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from VarSimu
        S += super(VarLoad, self).__sizeof__()
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from VarSimu
        VarLoad_dict = super(VarLoad, self).as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        VarLoad_dict["__class__"] = "VarLoad"
        return VarLoad_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        # Set to None the properties inherited from VarSimu
        super(VarLoad, self)._set_None()



# -*- coding: utf-8 -*-

from numpy import pi, sin, arcsin


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotCirc
        A SlotCirc object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]
    """

    R0 = ((self.W0 / 2) ** 2 + self.H0 ** 2) / (2 * self.H0)
    alpha = float(2 * arcsin(self.W0 / (2 * R0)))
    # Area of the full circle
    S1 = pi * (R0 ** 2)
    # Area of the top Arc
    S2 = (R0 ** 2.0) / 2.0 * (alpha - sin(alpha))
    # Arc of the bore
    alpha_op = self.comp_angle_opening()
    S3 = (self.get_Rbo() ** 2.0) / 2.0 * (alpha_op - sin(alpha_op))

    if self.is_outwards():
        return S1 - S2 - S3
    else:
        return S1 - S2 + S3



# -*- coding: utf-8 -*-


def get_lam_list_label(self):
    """Returns the ordered (from internal to external) list of lamination labels
    corresponding to machine.get_lam_list(is_int_to_ext=True, key=None)

    Parameters
    ----------
    self : Machine
        Machine object

    Returns
    -------
    label_list : list
        Ordered lamination list, for abstract Machine objects list will be empty
    """

    lam_list = self.get_lam_list(is_int_to_ext=True, key=None)

    label_list = []
    IDs = 0
    IDr = 0
    for lam in lam_list:
        if lam.is_stator:
            label_list.append("Stator_" + str(IDs))
            IDs += 1
        else:
            label_list.append("Rotor_" + str(IDr))
            IDr += 1

    return label_list



# -*- coding: utf-8 -*-

from numpy import sin, pi


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : Slot19
        A Slot19 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    Rbo = self.get_Rbo()

    # Wint is the with at the top of the opening
    alpha0 = self.comp_angle_opening()
    alpha1 = self.comp_angle_bottom()

    if self.is_outwards():
        R1 = Rbo + self.H0
    else:
        R1 = Rbo - self.H0

    W0 = 2 * Rbo * sin(alpha0 / 2)
    W1 = 2 * R1 * sin(alpha1 / 2)

    Str = 1 / 2 * (W0 + W1) * self.H0
    S0 = Rbo ** 2 / 2 * (alpha0 - sin(alpha0))
    S1 = R1 ** 2 / 2 * (alpha1 - sin(alpha1))

    return Str + S1 - S0



# -*- coding: utf-8 -*-

from numpy import abs as np_abs, arcsin, exp, pi, sin


def _comp_W(self):
    """Compute W1 and W2 method (cste tooth compatibility)

    Parameters
    ----------
    self : SlotW23
        A SlotW23 object
    Zs : int
        Number of Slot

    Returns
    -------
    None

    """
    Rbo = self.get_Rbo()

    # To compute the slot width we use the fact that
    # alpha_tooth + alpha_slot = slot_pitch
    slot_pitch = 2 * pi / self.Zs

    # H0 and H1 are radial
    if self.is_outwards():
        R1 = Rbo + self.H1 + self.H0
    else:
        R1 = Rbo - self.H1 - self.H0

    alpha_T1 = float(arcsin(self.W3 / (2 * R1)))
    alpha_S1 = slot_pitch - 2 * alpha_T1
    self.W1 = 2 * sin(alpha_S1 / 2) * R1

    # H2 is parallel to the tooth axis
    if self.is_outwards():
        R2 = np_abs(R1 * exp(1j * alpha_T1) + self.H2)
    else:
        R2 = np_abs(R1 * exp(1j * alpha_T1) - self.H2)

    alpha_T2 = float(arcsin(self.W3 / (2 * R2)))
    alpha_S2 = slot_pitch - 2 * alpha_T2
    self.W2 = 2 * sin(alpha_S2 / 2) * R2
